__author__ = 'churas'

import os
import logging

from d3r.celpp.task import D3RTask
from d3r.celpp.task import D3RParameters
from d3r.celpp.proteinligprep import ProteinLigPrepTask
from d3r.celpp.dataimport import DataImportTask
from d3r.celpp.participant import ParticipantDatabaseFromCSVFactory
from d3r.celpp import util
from d3r.celpp.task import SmtpEmailer
from d3r.celpp.task import Attachment


logger = logging.getLogger(__name__)


class PathNotDirectoryError(Exception):
    """Path is not a directory Error
    """
    pass


class EvaluationTaskFactory(object):
    """Factory class to generate ScoringTask objects

       This factory examines a celpp week directory for
       all docking tasks.  The code then generates
       ScoringTask objects for all eligible docking tasks
    """
    prep = ProteinLigPrepTask('/foo', D3RParameters())
    DOCKSTAGE = prep.get_stage() + 1
    DOCKSTAGE_PREFIX = (D3RTask.STAGE_DIRNAME_PREFIX + '.' +
                        str(DOCKSTAGE) + '.')
    SCORING_SUFFIX = 'evaluation'
    WEB_DATA_SUFFIX = 'webdata'

    def __init__(self, path, theargs):
        """Constructor
        """
        self.set_path(path)
        self.set_args(theargs)

    def set_args(self, theargs):
        """ Sets args
        :param theargs: arguments to set
        """
        self._args = theargs

    def get_args(self):
        """Gets args passed into constructor or via set_args()
        """
        return self._args

    def set_path(self, path):
        """Sets path used to look for docking tasks
        """
        self._path = path

    def get_path(self):
        """Gets path used to look for docking tasks
        """
        return self._path

    def _get_participant_database(self):
        """Creates `ParticipantDatabase`
        :returns: ParticipantDatabase
        """
        dimport = DataImportTask(self.get_path(), self.get_args())
        csvfile = dimport.get_participant_list_csv()
        pfac = ParticipantDatabaseFromCSVFactory(csvfile)
        return pfac.get_participant_database()

    def _get_value_of_replytoaddress(self):
        """Attempts to get value of replytoaddress from `get_args()`
        :returns None if replytoaddress is None or does not exist in object
        """
        try:
            return self.get_args().replytoaddress
        except AttributeError:
            logger.debug('replytoaddress not set in get_args()')
            return None

    def _update_priorities_of_tasks(self, etasks,
                                    participant_db):
        """Updates priorities for EvaluationTask objects in
           list of etask objects from values obtained in
           participant_db ParticipantDatabase
           :param etasks: list of EvaluationTask objects, it is assumed to
                          never be None
           :param participant_db: ParticipantDatabase object, assumed never
                                  to be none
           :returns: same list of EvaluationTask objects with priority set
        """
        for task in etasks:
            guid = task.get_guid_for_task()
            if guid is not None:
                logger.debug('Looking for participant with guid: ' + guid)
                p = participant_db.get_participant_by_guid(guid)
                if p is not None:
                    task.set_priority(p.get_priority())
                    logger.debug('Setting priority for ' + guid +
                                 ' to ' + str(task.get_priority()))
                else:
                    logger.debug('No participant found to match guid: ' + guid)
        return etasks

    def _sort_tasks_by_participant_priority(self, etasks,
                                            participant_db):
        """Sorts `EvaluationTask` objects in `etasks` by priority
           set for participants in `participant_db` The sorting
           goes as follows. Participants with highest get_priority()
           value go first, identical priority ordering is arbitrary
           and any EvaluationTasks without priority are put at end
           of list in arbitrary order
        :returns: list of sorted EvaluationTask objects
        """
        if etasks is None:
            logger.debug('No EvaluationTasks to sort')
            return etasks

        if participant_db is None:
            logger.warning('Participant Database is None, cannot sort')
            return etasks

        # update priorities of etasks
        updatedtasks = self._update_priorities_of_tasks(etasks, participant_db)
        updatedtasks.sort(reverse=True, key=lambda task: task.get_priority())
        return updatedtasks

    def get_evaluation_tasks(self):
        """Generate EvaluationTasks

           This method examines the path directory
           set via the constructor or set_path() method
           for all stage 4 tasks excluding tasks
           that end with 'webdata'  A EvaluationTask
           object is created for each of these tasks
           and returned in a list.
           :return: list of EvaluationTask objects or empty list if none found
        """
        path = self.get_path()
        logger.debug('Examining ' + path + ' for docking tasks')
        if not os.path.isdir(path):
            raise PathNotDirectoryError(path + ' is not a directory')
        scoring_tasks = []

        path_list = os.listdir(path)

        participant_db = self._get_participant_database()
        emailer = EvaluationEmailer(participant_db,
                                    self._get_value_of_replytoaddress())

        for entry in path_list:
            logger.debug('Checking if ' + entry + ' is a docking task')
            full_path = os.path.join(path, entry)
            if os.path.isdir(full_path):
                if entry.startswith(EvaluationTaskFactory.DOCKSTAGE_PREFIX):
                    if entry.endswith(EvaluationTaskFactory.WEB_DATA_SUFFIX):
                        logger.debug('Skipping ' + entry + ' due to suffix')
                        continue

                    # we have a valid docking path
                    docktask = D3RTask(path, self.get_args())
                    docktask.set_stage(EvaluationTaskFactory.DOCKSTAGE)
                    docktask.set_name(entry[
                                      len(EvaluationTaskFactory
                                          .DOCKSTAGE_PREFIX):])
                    stask = EvaluationTask(path,
                                           docktask.get_name() + '.' +
                                           EvaluationTaskFactory.
                                           SCORING_SUFFIX,
                                           docktask,
                                           self.get_args())
                    if stask.can_run():
                        logger.debug('Adding task ' + stask.get_name())
                        stask.set_evaluation_emailer(emailer)
                        scoring_tasks.append(stask)
                    else:
                        if stask.get_error() is None:
                            logger.debug(stask.get_name() + ' cannot be' +
                                         ' added, no error though')
                        else:
                            logger.debug(stask.get_name() + ' cannot be' +
                                         ' added : ' + stask.get_error())

        return self._sort_tasks_by_participant_priority(scoring_tasks,
                                                        participant_db)


class EvaluationEmailer(object):
    """Sends evaluation email
    """
    def __init__(self, participant_database, reply_to_address,
                 smtp='localhost', smtpport=25):
        self._participantdatabase = participant_database
        self._alt_smtp_emailer = None
        self._msg_log = None
        self._reply_to_address = reply_to_address
        self._smtp = smtp
        self._smtpport = smtpport

    def set_alternate_smtp_emailer(self, emailer):
        """Sets alternate smtp emailer
        """
        self._alt_smtp_emailer = emailer

    def _get_smtp_emailer(self):
        """Gets `SmtpEmailer` object or alternate
        """
        if self._alt_smtp_emailer is not None:
            return self._alt_smtp_emailer
        return SmtpEmailer(smtp_host=self._smtp,
                           port=self._smtpport)

    def _append_to_message_log(self, msg):
        if self._msg_log is None:
            self._msg_log = str(msg)
            return
        self._msg_log += str(msg)

    def get_message_log(self):
        """Gets the error log for this object
        :returns: Error log as string
        """
        return self._msg_log

    def _get_external_submitter_email(self, etask):
        """Extracts external submitter email from ParticipantDatabase

        :return: list of external submitter email or emails ot None if
        not found
        """
        # need to get guid from task name stage.#.GUID.extsubmission
        guid = etask.get_guid_for_task()
        if guid is None:
            logger.error('guid is None')
            self._append_to_message_log('\nUnable to extract guid\n')
            return None

        # then need to get email for guid
        part = self._participantdatabase.get_participant_by_guid(guid)

        if part is None:
            logger.error('No participant found with guid: ' + guid)
            self._append_to_message_log('\nNo participant found with guid: ' +
                                        guid + '\n')
            return None

        if part.get_email() is None:
            logger.error('Email not set for participant')
            self._append_to_message_log('\nEmail not set for participant\n')
            return None

        return part.get_email().split(',')

    def _get_reply_to_address(self, from_address):
        """Gets reply to address
        :returns: reply to email address
        """
        if self._reply_to_address is None:
            return from_address
        return self._reply_to_address

    def _generate_external_submission_email_body(self, etask):
        """Creates body of email and subject
        :returns subject,body: as strings
        """
        weekno = util.get_celpp_week_number_from_path(etask.get_path())

        msg = 'Dear CELPP Participant,\n\nHere are your docking ' \
              'evaluation results (RMSD, Angstroms) for CELPP week ' +\
              str(weekno) + '\n\n'
        msg += etask.get_evaluation_summary()

        msg += '\n\nSincerely,\n\nCELPP Automation'

        guid = etask.get_guid_for_task()
        subject = (D3RTask.SUBJECT_LINE_PREFIX +
                   'Week ' + str(weekno) + ' evaluation results for ' +
                   str(guid))
        return subject, msg

    def send_evaluation_email(self, etask):
        """Sends evaluation email appending issues to message log
        """

        # clear message log fix for issue #99
        self._msg_log = None
        if etask is None:
            logger.error('Task passed in is None')
            self._append_to_message_log('\nTask passed in is None\n')
            return

        if etask.is_external_submission() is False:
            logger.debug('Not an external submission, just returning')
            self._append_to_message_log('\nNot an external submission\n')
            return

        if self._participantdatabase is None:
            logger.error('Participant Database is None')
            self._append_to_message_log('\nParticipant database is None '
                                        'cannot send'
                                        ' docking evaluation email!!!\n')
            return
        try:
            emailer = self._get_smtp_emailer()

            to_list = self._get_external_submitter_email(etask)
            if to_list is None:
                logger.debug('No external submitter email, just returning')
                return

            subject, msg = self\
                ._generate_external_submission_email_body(etask)

            from_addr = emailer.generate_from_address_using_login_and_host()

            reply_to = self._get_reply_to_address(from_addr)

            rmsd = Attachment(etask.get_rmsd_txt(), 'rmsd.txt')

            emailer.send_email(from_addr, to_list, subject, msg,
                               reply_to=reply_to,
                               attachments=[rmsd])
            self._append_to_message_log('\nSent evaluation email to: ' +
                                        ", ".join(to_list) + '\n')
        except Exception as e:
            logger.exception('Caught exception')
            self._append_to_message_log('\nCaught exception trying to email '
                                        'participant : ' + str(e) + '\n')


class EvaluationTask(D3RTask):
    """Performs evaluation

    """

    FINAL_LOG = 'final.log'
    RMSD_TXT = 'RMSD.txt'
    RMSD_PICKLE = 'RMSD.pickle'
    EXT_SUBMISSION_SUFFIX = '.extsubmission'
    SCORE_DIR = 'score'
    COMPLEX_SUFFIX = '_complex.pdb'

    PDB_FILES = [SCORE_DIR + os.sep + 'crystal.pdb']

    def __init__(self, path, name, docktask, args):
        super(EvaluationTask, self).__init__(path, args)
        self.set_name(name)
        self.set_stage(EvaluationTaskFactory.DOCKSTAGE + 1)
        self.set_status(D3RTask.UNKNOWN_STATUS)
        self._docktask = docktask
        self._emailer = None
        self._priority = 0

    def set_priority(self, priority):
        """Sets priority of when evaluation task should be run
        :param priority: int denoting priority, higher values mean
                         higher priority and 0 is usually default
        """
        self._priority = priority

    def get_priority(self):
        """Gets priority of this EvaluationTask
        :returns: int denoting priority
        """
        return self._priority

    def set_evaluation_emailer(self, emailer):
        self._emailer = emailer

    def is_external_submission(self):
        """Checks if this evaluation task is analyzing an external submission
        The check is done by looking at suffix of name to see if it matches
        `ExternalDataSubmissionTask.EXT_SUBMISSION_SUFFIX`
        :returns: True for yes otherwise False
        """
        if self._docktask is None:
            return False

        if self._docktask.get_name().endswith(
                EvaluationTask.EXT_SUBMISSION_SUFFIX):
            return True
        return False

    def get_rmsd_txt(self):
        """Returns full path to RMSD.txt file
        :returns: full path to RMSD.txt file
        """
        return os.path.join(self.get_dir(), EvaluationTask.RMSD_TXT)

    def get_rmsd_pickle(self):
        """Returns full path to RMSD.pickle file
        :returns: full path to RMSD.pickle file
        """
        return os.path.join(self.get_dir(), EvaluationTask.RMSD_PICKLE)

    def get_evaluation_summary(self):
        """Parses RMSD.txt and generates human readable summary
        evaluating docking
        :returns: report of docking as string.
        """
        rmsd = self.get_rmsd_txt()

        start_line = '\nEvaluation of docking\n=====================\n'

        if not os.path.isfile(rmsd):
            return start_line + 'No ' + rmsd + ' file found.\n'

        try:
            f = open(rmsd, 'r')
            summary = f.read()
            f.close()
            return start_line + summary + '\n'
        except Exception as e:
            logger.exception('Caught exception')
            return (start_line + 'Unable to generate evaluation summary (' +
                    str(e) + ')\n')

    def get_guid_for_task(self):
        """Examines task name removing external submission suffix
           to hopefully obtain guid
        :returns: guid from task name
        """
        if self._docktask is None:
            logger.error('Parent DockTask is None, so guid is'
                         ' NOT extractable')
            return None

        return self._docktask.get_name()\
            .replace(EvaluationTask.EXT_SUBMISSION_SUFFIX, '')

    def get_uploadable_files(self):
        """Returns list of files that can be uploaded to remote server

           List will contain these files if found on the file system
           plus stderr/stdout files

           RMSD.txt
           RMSD.pickle
           final.log
           pbdid/score/crystal.pdb
           pbdid/score/*_complex.pdb

           :returns: list of files that can be uploaded
        """
        # get the stderr/stdout files
        file_list = super(EvaluationTask, self).get_uploadable_files()

        out_dir = self.get_dir()

        try:
            final_log = os.path.join(out_dir, EvaluationTask.FINAL_LOG)
            if os.path.isfile(final_log):
                file_list.append(final_log)

            rmsd = self.get_rmsd_txt()
            if os.path.isfile(rmsd):
                file_list.append(rmsd)

            rmsdpickle = self.get_rmsd_pickle()
            if os.path.isfile(rmsdpickle):
                file_list.append(rmsdpickle)

            for entry in os.listdir(out_dir):
                full_path = os.path.join(out_dir, entry)
                if not os.path.isdir(full_path):
                    continue

                logger.debug('Looking for pdb files in ' + full_path)
                for pdb_name in EvaluationTask.PDB_FILES:
                    pdb = os.path.join(full_path, pdb_name)
                    if os.path.isfile(pdb):
                        file_list.append(pdb)
                score_dir = os.path.join(full_path,
                                         EvaluationTask.SCORE_DIR)
                if os.path.isdir(score_dir):
                    for score_entry in os.listdir(score_dir):
                        if score_entry.endswith(EvaluationTask.COMPLEX_SUFFIX):
                            pdb_f_path = os.path.join(score_dir, score_entry)
                            if os.path.isfile(pdb_f_path):
                                file_list.append(pdb_f_path)
        except OSError:
            logger.exception('Caught exception looking for pbdid folders')
        return file_list

    def can_run(self):
        """Determines if task can actually run

           This method first verifies the docking task
           has `D3RTask.COMPLETE_STATUS` for
           status.  The method then verifies this task does
           not already exist.  If above is not true then self.set_error()
           is set with information about the issue
           :return: True if can run otherwise False
        """
        self._can_run = False
        self._error = None
        self._docktask.update_status_from_filesystem()
        if self._docktask.get_status() != D3RTask.COMPLETE_STATUS:
            logger.info('Cannot run ' + self.get_name() + ' task ' +
                        'because ' + self._docktask.get_name() + ' task' +
                        ' has a status of ' + self._docktask.get_status())
            self.set_error(self._docktask.get_name() + ' task has ' +
                           self._docktask.get_status() + ' status')
            return False

        # check this task is not complete and does not exist

        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def run(self):
        """Runs EvaluationTask after verifying dock was good

           Method requires can_run() to be called before hand with
           successful outcome
           Otherwise method invokes D3RTask.start then this method
           creates a directory and invokes evaluation script.  Upon
           completion results are analyzed and success or error status
           is set appropriately and D3RTask.end is invoked
           """
        super(EvaluationTask, self).run()

        if self._can_run is False:
            logger.debug(
                self.get_dir_name() + ' cannot run cause _can_run flag '
                                      'is False')
            return

        try:
            logger.debug('evaluation set to ' +
                         self.get_args().evaluation)
        except AttributeError:
            self.set_error('evaluation not set')
            self.end()
            return

        try:
            logger.debug('pdbdb set to ' +
                         self.get_args().pdbdb)
        except AttributeError:
            self.set_error('pdbdb not set')
            self.end()
            return

        try:
            evaltimeout = self.get_args().evaluationtimeout
            logger.debug('Setting evaluation timeout to ' +
                         str(evaltimeout))
        except AttributeError:
            evaltimeout = None

        try:
            killdelay = self.get_args().evaluationtimeoutkilldelay
            logger.debug('Setting evaluation kill delay to ' +
                         str(killdelay))
        except AttributeError:
            killdelay = 60

        #
        # --pdbdb <path to pdb.extracted> --dockdir <stage.4.glide> \
        # --outdir <path to stage.5.glide.evaluation>
        #
        cmd_to_run = (self.get_args().evaluation + ' --pdbdb ' +
                      self.get_args().pdbdb + ' --dockdir ' +
                      self._docktask.get_dir() +
                      ' --outdir ' + self.get_dir())

        eval_name = os.path.basename(self.get_args().evaluation)

        self.run_external_command(eval_name, cmd_to_run,
                                  False,
                                  timeout=evaltimeout,
                                  kill_delay=killdelay)

        # attempt to send evaluation email
        try:
            self._emailer.send_evaluation_email(self)
            self.append_to_email_log(self._emailer.get_message_log())
        except Exception as e:
            logger.exception('Caught exception trying to send evaluation '
                             'email')
            self.append_to_email_log('Caught exception trying to send '
                                     'evaluation email ' + str(e) + '\n')

        # assess the result
        self.end()



#!/usr/bin/env python

import commands
import os
import glob
import logging
import time

logger = logging.getLogger()
logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level   = logging.INFO )


def grid (grid_center, prep_pro, prefix):
    #protein_name = prep_pro.split(".")[0]
    out_grid = prefix + ".zip"
    grid_in  = prefix +  "_grid.in"
    grid_1 = "GRID_CENTER \t %s \n"%grid_center
    grid_2 = "GRIDFILE \t %s \n"%out_grid
    grid_3 = "INNERBOX \t 10, 10, 10 \n"
    grid_4 = "OUTERBOX \t 30, 30, 30 \n"
    grid_5 = "RECEP_FILE \t %s \n"%prep_pro
    f = open (grid_in, "w")
    for grid_line in (grid_1, grid_2, grid_3, grid_4, grid_5):
        f.writelines(grid_line)
    f.close()
    commands.getoutput("$SCHRODINGER/glide -WAIT %s"%grid_in)
    if os.path.isfile(out_grid):
        return out_grid
    else:
        return False

def dock(ligand_file, out_grid, protein_title, precision='SP'):
    allowed_precisions = ['SP','XP']
    if not(precision in allowed_precisions):
        logging.info('Invalid precision setting %s. Must be one of %r. Switching precision to %s' %(precision, allowed_precisions, allowed_precisions[0]))
        precision = allowed_precisions[0]
    dock_in = protein_title + "_dock.in"
    dock_lines = []
    dock_lines += ["GRIDFILE \t %s \n"%out_grid]
    dock_lines += ["LIGANDFILE \t %s \n"%ligand_file]
    dock_lines += ["POSES_PER_LIG \t 10 \n"]
    if precision == 'SP':
        dock_lines += ["PRECISION \t XP \n"]
    elif precision == 'XP':
        dock_lines += ["PRECISION \t XP \n"]
        dock_lines += ["POSTDOCK_XP_DELE \t 0.5 \n"]
        dock_lines += ["EXPANDED_SAMPLING \t True \n"]
        dock_lines += ["WRITE_XP_DESC \t False \n"]
    f = open(dock_in, "w")
    for dock_line in dock_lines:
        f.writelines(dock_line)
    f.close()
    out_dock_file = protein_title + "_dock_pv.maegz"
    commands.getoutput("$SCHRODINGER/glide -WAIT %s"%dock_in)
    return out_dock_file

def main_glide (stage_3_result, stage_4_working, update= True, usexp=False):
    if usexp:
        precision = 'XP'
    else:
        precision = 'SP'
    os.chdir(stage_4_working)
    current_dir = os.getcwd()
    dockable_paths = []
    for all_pdb_path in os.walk(stage_3_result): 
       if all_pdb_path[0] != stage_3_result:
            dockable_paths.append(all_pdb_path[0])
    for dockable_path in dockable_paths:
        commands.getoutput("cp -r %s %s" %(dockable_path, stage_4_working))
        target_name = os.path.basename(dockable_path)
        os.chdir(target_name)
        current_dir_layer_2 = os.getcwd()
        ##################
        #Do the docking here
        
        #1, get the center
        try:
            grid_center = open("center.txt","r").readlines()[0]
            logging.info("=============Start working on this case:%s=========="%target_name)
        except:
            logging.info("Fatal error: Unable to find the center file for this case %s"%target_name)
            os.chdir(current_dir)
            continue
            
        # Get the candidate protein names in this directory
        candidate_proteins = glob.glob('./*-????_????_prepared.mae')
        logging.info('Found candidates %r' %(candidate_proteins))

        # Get the ligand names in this directory
        ligand_maes = glob.glob('lig_*_prepared.mae')
        ligand_maes = [i for i in ligand_maes if not "unprep" in i]
        if len(ligand_maes) == 0:
            logging.info('No ligand files found for target %s. Skipping.' %(dockable_path))
            os.chdir(current_dir)
            continue
        if len(ligand_maes) > 1:
            logging.info('Multiple ligand files found for target %s (found ligands %r). The workflow currently should only be sending one ligand. Skipping. ' %(dockable_path, ligand_maes))
            os.chdir(current_dir)
            continue
            
        #for possible_protein in ("LMCSS.maegz", "SMCSS.maegz", "hiResHolo.maegz", "hiResApo.maegz"):
        for candidate_protein in candidate_proteins:
            candidate_prefix = candidate_protein.replace('_prepared.mae','')

            #if os.path.isfile(candidate_protein):
            logging.info( "Working on this receptor: %s"%candidate_prefix )
            if not os.path.isdir(candidate_prefix):
                os.mkdir(candidate_prefix)
            os.chdir(candidate_prefix)
            ##################
            #do docking inside
            commands.getoutput("cp ../%s ."%candidate_protein)

            #generate grid
            logging.info("Trying to get the grid file...")
            if update: 
                out_grid = grid(grid_center, candidate_protein, candidate_prefix) 
                #time.sleep(200)
            else:
                #check if there is old grid
                out_grid = candidate_prefix + ".zip"
                if not os.path.isfile(out_grid):
                    out_grid = grid(grid_center, candidate_protein, candidate_prefix)
                    #time.sleep(200)
            logging.info("Finished grid generation...")
            
            ## Dock each ligand
            for ligand_mae in ligand_maes:
                commands.getoutput("cp ../%s ." %(ligand_mae))
                logging.info("Trying to dock...")
                if update:
                    out_dock = dock("ligand.mae", out_grid, candidate_prefix, precision=precision)
                else:
                    out_dock = candidate_prefix + "_dock_pv.maegz"
                    if not os.path.isfile(out_dock):
                        out_dock = dock(ligand_mae, out_grid, candidate_prefix, precision=precision)
                logging.info("Finished docking, beginning post-docking file conversion")
                
                # Split the results into receptor and poses
                intermediate_prefix = '%s_postdocking' %(candidate_prefix)
                receptorMae = intermediate_prefix+'_receptor1.mae'
            
            
                ## Split into a receptor mae file and one ligand mae for each pose
                commands.getoutput('$SCHRODINGER/run split_structure.py -m ligand -many_files %s %s.mae' %(out_dock, intermediate_prefix))
            
                ## Convert the receptor mae into pdb
                # This pdb is one of the final outputs from docking
                outputReceptorPdb = '%s_docked.pdb' %(candidate_prefix)
                commands.getoutput('$SCHRODINGER/utilities/structconvert %s %s' %(receptorMae, outputReceptorPdb))
            
                ## Convert the ligand maes into mols
                docked_ligand_maes = glob.glob('./%s_ligand?.mae' %(intermediate_prefix))
                for docked_ligand_mae in docked_ligand_maes:
                    docked_ligand_mol = docked_ligand_mae.replace('.mae','.mol') 
                    commands.getoutput('$SCHRODINGER/utilities/structconvert %s %s' %(docked_ligand_mae, docked_ligand_mol))

            # Copy the top-ranked ligand mol to be one of the final outputs from this step
            top_ligand_mol = intermediate_prefix+'_ligand1.mol'
            output_ligand_mol = '%s_docked.mol' %(candidate_prefix)
            commands.getoutput('cp %s %s' %(top_ligand_mol, output_ligand_mol))
            commands.getoutput('cp %s ../' %(output_ligand_mol))
            commands.getoutput('cp %s ../' %(outputReceptorPdb))
      
            ##################
            os.chdir(current_dir_layer_2)
        os.chdir(current_dir)
        ##################

if ("__main__") == (__name__):
    from argparse import ArgumentParser
    parser = ArgumentParser()
    parser.add_argument("-s", "--structuredir", metavar="PATH", help = "PATH where we can find the proteinligprep output")
    parser.add_argument("-o", "--outdir", metavar = "PATH", help = "PATH where we will put the docking output")
    parser.add_argument("-u", "--update", action = "store_true",  help = "Update the docking result", default = False) 
    parser.add_argument("-x", "--usexp", action = "store_true",  help = "Perform docking using glide XP instead of SP", default = False) 
    logger = logging.getLogger()
    logging.basicConfig( format  = '%(asctime)s: %(message)s', datefmt = '%m/%d/%y %I:%M:%S', filename = 'final.log', filemode = 'w', level = logging.INFO )
    opt = parser.parse_args()
    proteinligprep_dir = opt.structuredir
    dock_result_dir = opt.outdir
    update = opt.update
    usexp = opt.usexp
    #running under this dir
    running_dir = os.getcwd()
    main_glide(proteinligprep_dir, 
               dock_result_dir, 
               update = update,
               usexp = usexp)
    #move the final log file to the result dir
    log_file_path = os.path.join(running_dir, 'final.log')
    commands.getoutput("mv %s %s"%(log_file_path, dock_result_dir))



__author__ = 'robswift'

import logging
from Bio import Alphabet
from d3r.filter.filtering_sets import stabilisers, buffers, ions, excipients

logger = logging.getLogger(__name__)


class BaseFilter(object):
    """

    """

    def __init__(self, query):
        """
        The filter class is initiated with unique target structures (target objects) contained in the targets list. If
        the target list was filled by reading new_release_structure_sequence.tsv, and
        new_release_structure_nonpolymer.tsv, there is one target object for each of the unique wwPDB ids in
        new_release_sequence.tsv. And if their are corresponding ligands in new_release_structure_nonpolymer.tsv,
        ligand information is also included in the target object.
        :param query: (list) a list of unique target objects
        """
        self.query = query


class QueryFilter(BaseFilter):
    """

    """

    dockable_ligand_threshold = 1
    sequence_threshold = 1

    def __init__(self, *args, **kwargs):
        super(QueryFilter, self).__init__(*args, **kwargs)

    def filter_by_inchi_error(self):
        """
        Remove target structures whose dockable ligand(s) produced an error on attempting to produce a rdkit.Chem.rdchem
        Mol object from the input InChI.
        """
        if self.query.inchi_error:
            self.query.set_reason(9)

    def filter_apo(self):
        """
        Remove queries structures without dockable ligands. I.e. If an input queries structure contains no dockable
        ligands, it is labeled 'tosser'.
        """
        if self.query.dock_count == 0:
            self.query.set_reason(6)

    def filter_by_sequence_count(self, threshold = None):
        if threshold == None:
            threshold = QueryFilter.sequence_threshold
            if self.query.sequence_count > threshold:
                self.query.set_reason(3)

    def filter_by_sequence_type(self):
        for seq_rec in self.query.sequences:
            if not Alphabet._verify_alphabet(seq_rec.seq):
                self.query.set_reason(11)
                break

    def filter_by_dockable_ligand_count(self, threshold=None):
        """
        Remove query structures with too many dockable ligands. The default behavior removes query structures with
        more than one dockable ligand. Each query object whose number of dockable ligands is greater than the input
        threshold will be labeled'tosser'.
        :param threshold: (int) threshold value for the number of unique dockable ligands in each query. Default = 1.
        """
        if threshold is None:
            threshold = QueryFilter.dockable_ligand_threshold
            if self.query.dock_count > threshold:
                self.query.set_reason(4)


class HitFilter(BaseFilter):
    """
    Initiated with a list of target objects, this class can run a number of different filtering operations that will
    label the Hit objects 'tosser' if they fail to pass any one of the filtering criteria. After initializing and
    running the filtering operations, the method get_filtered_query() is called, and the filtered target_list is
    returned. For example
        filter = filter(target_list)
        filter.filter_by_target_chains(5)
        filter.filter_by_experiment('X-RAY DIFFRACTION')
        filtered_list = filter.get_filtered_query()
    """
    # default filtering values
    identity_threshold = 0.95
    coverage_threshold = 0.90
    sequence_threshold = 2 
    #dockable_ligand_threshold = 1
    #sliu change to big number to "turn off" the dockable ligand filter 08/04  
    dockable_ligand_threshold = 10
    method = 'x-ray diffraction'

    def __init__(self, *args, **kwargs):
        super(HitFilter, self).__init__(*args, **kwargs)

    def filter_by_identity(self, threshold=None):
        """
        Remove hit structures whose sequences are insufficiently identical to the query sequence. This is measured by
        percent identity, where percent identity is the fraction of the BLAST alignment identical to the query. In the
        case of multi-chain BLAST hits, all of the chains must satisfy the threshold, or the hit is labeled a tosser.
        The default behavior removes those hit structures whose sequences have a percent identity less than 95%. Each
        hit structure with one or more sequences whose percent identity is less than the input threshold will be
        labeled 'tosser'. If all of the hit structures of a given target are labeled 'tosser', then the target
        structure is also labeled 'tosser.'
        :param threshold: (float) percent identity threshold. Default = 0.95
        """
        logger.debug('In filter_by_identity()')
        if threshold is None:
            threshold = HitFilter.identity_threshold
            for hit in self.query.hits:
                query_alignments = []
                for chain in [chain for chain in hit.sequences if chain.blast_hit]:
                    chain.sort_by_identity()
                    query_alignments.append(chain.query_alignments[0])
                if len([query_alignment for query_alignment in query_alignments if query_alignment.identity
                       < threshold]) > 0:
                    hit.set_reason(1)
            if len([hit for hit in self.query.hits if hit.triage]) == len(self.query.hits):
                self.query.set_reason(8)

    def filter_by_coverage(self, threshold = None):
        """
        Remove hit structures whose sequences don't adequately cover the query sequence. This is measured by percent
        coverage, where percent coverage is the fraction of query sequence covered by the alignment. In the case of
        multi-chain BLAST hits, all of the chains must satisfy the threshold, or the hit is labeled a tosser. The
        default behavior removes those hit structures whose sequences have a percent identity less than 90%. Each
        hit structure with one or more sequences whose percent coverage is less than the input threshold will be
        labeled 'tosser'. If all of the hit structures of a given query are labeled 'tosser', then the query is also
        labeled 'tosser.'
        :param threshold: (float) percent coverage threshold. Default = 0.90
        """
        logger.debug('In filter_by_coverage()')
        # If the highest coverage for one or more chains is less than the threshold, then triage
        if threshold == None:
            threshold = HitFilter.coverage_threshold
            for hit in self.query.hits:
                query_alignments = []
                for chain in [chain for chain in hit.sequences if chain.blast_hit]:
                    chain.sort_by_coverage()
                    query_alignments.append(chain.query_alignments[0])
                if len([query_alignment for query_alignment in query_alignments if query_alignment.coverage
                        < threshold]) > 0:
                    hit.set_reason(2)
            if len([hit for hit in self.query.hits if hit.triage]) == len(self.query.hits):
                self.query.set_reason(8)

    def filter_by_sequence_count(self, threshold = None):
        """
        Remove hit structures with too many unique sequences. The default behavior removes hit structures with more than
        found unique sequences. Each hit object whose unique sequences number more than the input threshold will be
        labeled 'tosser'. If all of the hit objects of a given query are labeled 'tosser', then the query is also
        labeled 'tosser.'
        :param threshold: (int) threshold value for the number of unique sequences in each hit structure. Default = 4
        """
        logger.debug('In filter_by_sequence_count()')
        if threshold == None:
            threshold = HitFilter.sequence_threshold
            for hit in self.query.hits:
                #print "Check in the hit filter step sequence count:%s in this pdb id: %s"%(hit.sequence_count, hit.pdb_id)
                #raw_input()
                if hit.sequence_count > threshold:
                    hit.set_reason(3)
            if len([hit for hit in self.query.hits if hit.triage]) == len(self.query.hits):
                self.query.set_reason(8)

    def filter_apo(self):
        """
        If a blast search of the wwPDB using the query sequence only returns apo hit structures, then the query will
        be labeled 'tosser'.
        """
        logger.debug('In filter_apo()')
        if len([hit for hit in self.query.hits if hit.dock_count == 0]) == len(self.query.hits):
            for hit in self.query.hits:
                hit.set_reason(10)
            self.query.set_reason(5)

    def filter_by_method(self, method=None):
        """
        Removes hit structures that weren't determined by the input method type.
        Method types can include:
            'x-ray diffraction', 'solution nmr', 'solid-state nmr', 'electron microscopy', 'electron crystallography',
            'fiber diffraction', 'neutron diffraction', 'solution scattering'.
        :param method: (string) an experimental structure determination method used to solve wwPDB structures.
        Default = 'x-ray diffraction'
        """
        logger.debug('In filter_by_method()')
        if not method: method = HitFilter.method
        for hit in self.query.hits:
            if hit.exp_method != method.lower():
                hit.set_reason(7)
        if len([hit for hit in self.query.hits if hit.triage]) == len(self.query.hits):
            self.query.set_reason(8)

    def filter_by_dockable_ligand_count(self, threshold=None):
        """
        Remove query structures with too many dockable ligands. The default behavior removes query structures with
        more than one dockable ligand. Each query object whose number of dockable ligands is greater than the input
        threshold will be labeled'tosser'.
        :param threshold: (int) threshold value for the number of unique dockable ligands in each query. Default = 1.
        """
        logger.debug('In filter_by_dockable_ligand_count()')
        if threshold is None:
            threshold = HitFilter.dockable_ligand_threshold
            for hit in self.query.hits:
                #print "Check in the hit filter step dock_count%s in this pdb id:%s "%(hit.dock_count, hit.pdb_id)
                #raw_input()
                if hit.dock_count > threshold:
                    hit.set_reason(4)


class CandidateFilter(BaseFilter):
    """
    Filters the queries and hits that haven't been discarded by the PrimaryFilter. Specifically, for each remaining
    query object, the BLAST hits that satisfy the following criteria are identified
        1. The hit whose ligand is most similar to the ligand of the corresponding query
        2. The hit whose ligand is least similar to the ligand of the corresponding query
        3. The hit whose structure has the highest resolution
        4. The apo hit with the highest resolution
    """
    def __init__(self, *args, **kwargs):
        super(CandidateFilter, self).__init__(*args, **kwargs)

    def filter_for_most_similar(self):
        logger.debug('In filter_for_most_similar()')
        # sort the hit list by decreasing MCSS size and increasing resolution
        # the most similar will be at the front of the list, the least similar will be at the end of the list.
        logger.debug('In filter_for_most_similar()')
        hits = [hit for hit in self.query.hits if not hit.triage and hit.largest_mcss]
        if hits:
            hits.sort(key=lambda hit: (int(hit.largest_mcss.size), float(hit.resolution)), reverse=True)
            # hits[0].set_retain_reason(1)    # picks off the largest mcss with the highest resolution crystal structure
            lowest = hits[0].resolution     # lowest resolution
            largest = hits[0].largest_mcss.size  # largest maximum common substructure
            #print "Check the TTTTop largest mcss: %s and resolution: %s"%(largest, lowest)
            #raw_input()
            for hit in hits:
                #print "Check the largest mcss: %s and resolution: %s for this hit:%s "%(hit.largest_mcss.size ,hit.resolution, hit.pdb_id)
                #raw_input()
                if hit.resolution > lowest or hit.largest_mcss.size < largest:
                    break
                else:
                    #print "Check the largest mcss: %s and resolution: %s for this hit:%s "%(hit.largest_mcss.size ,hit.resolution, hit.pdb_id)
                    #raw_input()
                    hit.set_retain_reason(1)

    def filter_for_least_similar(self):
        logger.debug('In filter_for_least_similar()')
        # sort the hits by increasing MCSS size and increasing resolution
        hits = [hit for hit in self.query.hits if not hit.triage and hit.smallest_mcss]
        if hits:
            hits.sort(key=lambda hit: (int(hit.smallest_mcss.size), float(hit.resolution)))
            # hits[0].set_retain_reason(2)   # picks off the smallest mcss with the highest resolution crystal structure
            lowest = hits[0].resolution       # lowest resolution
            smallest = hits[0].smallest_mcss.size  # largest maximum common substructure
            for hit in hits:
                if hit.resolution > lowest or hit.smallest_mcss.size > smallest:
                    break
                else:
                    hit.set_retain_reason(2)

    def filter_holo(self):
        logger.debug('In filter_holo()')
        #sliu 01102017 fix the bug to skip the hit which don't have the mcss with query ligand
        hits = [hit for hit in self.query.hits if hit.resolution and not hit.triage and hit.dock_count > 0 and hit.largest_index]
        if hits:
            hits.sort(key=lambda hit: hit.resolution)
            lowest = hits[0].resolution
            for hit in hits:
                if hit.resolution > lowest:
                    break
                else:
                    hit.set_retain_reason(3)

    def filter_apo(self):
        """
        Hits are only considered apo if: i) the number of dockable ligands is 0 and ii) all the non-dockable ligands
        may only be buffers, ions, excipients, or stabilisers (the sets in filter.filtering_sets.py).
        """
        logger.debug('In filter_apo()')
        allowable_apo = set()
        allowable_apo.update(stabilisers)
        allowable_apo.update(buffers)
        allowable_apo.update(ions)
        allowable_apo.update(excipients)
        hits = [hit for hit in self.query.hits if hit.resolution and not hit.triage and hit.dock_count == 0]
        if hits:
            hits.sort(key=lambda hit: hit.resolution)
            lowest = hits[0].resolution
            for hit in hits:
                # check that the no. of non-dockable ligands that are either buffers, ions, excipients, or stabilizers
                # is identical to the number of non-dockable ligands.
                if len([lig for lig in hit.do_not_call if lig.resname in allowable_apo]) == len(hit.do_not_call):
                    if hit.resolution > lowest:
                        break
                    else:
                        hit.set_retain_reason(4)
    #add this new fuction to filter the highest tanimoto 0901 sliu
    def filter_for_highest_tanimoto(self):
        logger.debug('In filter_for_highest_tanimoto()')
        # sort the hit list by decreasing MCSS size and increasing resolution
        # the most similar will be at the front of the list, the least similar will be at the end of the list.
        hits = [hit for hit in self.query.hits if not hit.triage and hit.highest_tanimoto]
        if hits:
            hits.sort(key=lambda hit: (float(hit.highest_tanimoto.tanimoto), float(hit.resolution)), reverse=True)
            # hits[0].set_retain_reason(1)    # picks off the largest mcss with the highest resolution crystal structure
            lowest = hits[0].resolution     # lowest resolution
            highest = hits[0].highest_tanimoto.tanimoto  # highest tanimoto 
            #print "Check the TTTTop largest mcss: %s and resolution: %s"%(largest, lowest)
            for hit in hits:
                #print "Check the largest mcss: %s and resolution: %s for this hit:%s "%(hit.largest_mcss.size ,hit.resolution, hit.pdb_id)
                #raw_input()
                if hit.resolution > lowest or hit.highest_tanimoto.tanimoto < highest:
                    break
                else:
                    #print "Check the largest mcss: %s and resolution: %s for this hit:%s "%(hit.largest_mcss.size ,hit.resolution, hit.pdb_id)
                    #raw_input()
                    hit.set_retain_reason(5)



__author__ = 'churas'

"""
test_vinadocking
--------------------------------

Tests for `vinadocking` module.
"""

import unittest
import os
import os.path
import re


class TestCelppCommandLineScripts(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def get_list_of_celpp_scripts(self):
        d3r_dir = os.path.join(os.getcwd(), 'd3r')
        celppscripts = []
        for entry in os.listdir(d3r_dir):
            fullpath = os.path.join(d3r_dir, entry)
            if fullpath.endswith('__init__.py'):
                continue
            if os.path.isfile(fullpath):
                if fullpath.endswith('.py'):
                    celppscripts.append(fullpath)

        return celppscripts

    def test_every_script_starts_with_correct_shebang(self):
        """This test was added since a commit was recently made
           where the shebang was modified and had a typo in it
           causing all the scripts to fail
        """
        for script in self.get_list_of_celpp_scripts():
            f = open(script, 'r')
            val = re.sub('\n$', '', f.readline())
            self.assertTrue(re.match('#! *\/usr\/bin\/env +python', val),
                            ' invalid shebang line in file: ' + script +
                            ' : ' + val)
            f.close()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'churas'

import unittest
import tempfile
import shutil
import os
import tarfile
from mock import Mock

from d3r.celpp.extsubmission import ExternalDataSubmissionFactory
from d3r.celpp.extsubmission import ExternalDataSubmissionTask
from d3r.celpp.extsubmission import ChallengePackageDownloadError
from d3r.celpp.task import D3RParameters
from d3r.celpp import util
from d3r.celpp.task import D3RTask

"""
test_extsubmission
--------------------------------

Tests for `extsubmission` module.
"""


class TestExternalSubmission(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_externaldatasubmissionfactory_noftpconfig(self):
        params = D3RParameters()
        fac = ExternalDataSubmissionFactory('/foo', params)
        self.assertEqual(fac.get_file_transfer(), None)

    def test_externaldatasubmissionfactory_ftpconfig_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            figfile = os.path.join(temp_dir, 'foo')
            f = open(figfile, 'w')
            f.write('host blah.blah.com\n')
            f.write('user bob\n')
            f.write('pass ha\n')
            f.write('path /celppweekly/ha\n')
            f.write('challengepath /celppweekly/ha\n')
            f.flush()
            f.close()
            params = D3RParameters()
            params.ftpconfig = figfile
            fac = ExternalDataSubmissionFactory('/foo', params)
            self.assertTrue(fac.get_file_transfer() is not None)
        finally:
            shutil.rmtree(temp_dir)

    def test_externaldatasubmissionfactory_getter_setter(self):
        params = D3RParameters()
        params.ha = 'hi'
        fac = ExternalDataSubmissionFactory('/foo', params)
        self.assertEqual(fac.get_file_transfer(), None)
        self.assertEqual(fac.get_path(), '/foo')
        self.assertEqual(fac.get_args().ha, 'hi')

        fac.set_file_transfer('yo')
        self.assertEqual(fac.get_file_transfer(), 'yo')

    def test_get_challenge_package_results_file_name(self):
        params = D3RParameters()

        # test under non existant dir
        fac = ExternalDataSubmissionFactory('/foo', params)
        self.assertEqual(fac._get_challenge_package_results_file_name('xxx'),
                         'celpp_week0_0_dockedresults_xxx.tar.gz')

        # test under 2016 week 40
        temp_dir = tempfile.mkdtemp()
        try:
            year = os.path.join(temp_dir, '2016')
            week = os.path.join(year, util.DATA_SET_WEEK_PREFIX + '40')
            os.makedirs(week)
            fac = ExternalDataSubmissionFactory(week, params)
            rfname = fac._get_challenge_package_results_file_name('xxx')
            self.assertEqual(rfname,
                             'celpp_week40_2016_dockedresults_xxx.tar.gz')

        finally:
            shutil.rmtree(temp_dir)

    def test_get_submission_dirs_filetransfer_is_none(self):
        params = D3RParameters()
        try:
            fac = ExternalDataSubmissionFactory('/foo', params)
            fac._get_submission_dirs('ha')
            self.fail('Expected AttributeError')
        except AttributeError:
            pass

    def test_get_submission_dirs_list_is_none(self):
        params = D3RParameters()
        mockft = D3RParameters()
        mockft.list_dirs = Mock(return_value=None)
        fac = ExternalDataSubmissionFactory('/foo', params)
        fac.set_file_transfer(mockft)
        self.assertEqual(len(fac._get_submission_dirs('ha')), 0)
        mockft.list_dirs.assert_called_with('ha')

    def test_get_submissions_dirs_empty(self):
        params = D3RParameters()
        mockft = D3RParameters()
        mockft.list_dirs = Mock(return_value=[])
        fac = ExternalDataSubmissionFactory('/foo', params)
        fac.set_file_transfer(mockft)
        self.assertEqual(len(fac._get_submission_dirs('ha')), 0)
        mockft.list_dirs.assert_called_with('ha')

    def test_get_submission_dirs_one_dir(self):
        params = D3RParameters()
        mockft = D3RParameters()
        mockft.list_dirs = Mock(return_value=['somedir'])
        fac = ExternalDataSubmissionFactory('/foo', params)
        fac.set_file_transfer(mockft)
        dlist = fac._get_submission_dirs('ha')
        self.assertEqual(dlist[0], 'somedir')
        mockft.list_dirs.assert_called_with('ha')

    def test_get_submission_dirs_two_dirs(self):
        params = D3RParameters()
        mockft = D3RParameters()
        mockft.list_dirs = Mock(return_value=['somedir', 'blah'])
        fac = ExternalDataSubmissionFactory('/foo', params)
        fac.set_file_transfer(mockft)
        dlist = fac._get_submission_dirs('ha')
        self.assertEqual(dlist[0], 'somedir')
        self.assertEqual(dlist[1], 'blah')
        mockft.list_dirs.assert_called_with('ha')

    def test_get_challenge_data_package_file_raise_exception(self):
        params = D3RParameters()
        fac = ExternalDataSubmissionFactory('/foo', params)
        try:
            fac._get_challenge_data_package_file('/remote', 'dname')
            self.fail('Expected AttributeError')
        except AttributeError:
            pass

    def test_get_challenge_data_package_file_none_for_files(self):
        params = D3RParameters()
        mockft = D3RParameters()
        mockft.list_files = Mock(return_value=None)
        fac = ExternalDataSubmissionFactory('/foo', params)
        fac.set_file_transfer(mockft)
        dlist = fac._get_challenge_data_package_file('/remote', 'dname')
        self.assertEqual(dlist, None)
        mockft.list_files.assert_called_with('/remote/dname')

    def test_get_challenge_data_package_file_no_files(self):
        params = D3RParameters()
        mockft = D3RParameters()
        mockft.list_files = Mock(return_value=[])
        fac = ExternalDataSubmissionFactory('/foo', params)
        fac.set_file_transfer(mockft)
        dlist = fac._get_challenge_data_package_file('/remote', 'dname')
        self.assertEqual(dlist, None)
        mockft.list_files.assert_called_with('/remote/dname')

    def test_get_challenge_data_package_file_no_match(self):
        temp_dir = tempfile.mkdtemp()
        try:
            dr = ExternalDataSubmissionFactory.DOCKEDRESULTS
            year = os.path.join(temp_dir, '2016')
            week = os.path.join(year, 'dataset.week.13')
            os.makedirs(week)
            params = D3RParameters()
            mockft = D3RParameters()
            mockft.list_files = Mock(return_value=['hi', 'celpp_week13_2015_' +
                                                   dr + '_dname.tar.gz',
                                                   'celpp_week13_2016' + dr +
                                                   'yuck.tar.gz'])
            fac = ExternalDataSubmissionFactory(week, params)
            fac.set_file_transfer(mockft)
            dlist = fac._get_challenge_data_package_file('/remote', 'dname')
            self.assertEqual(dlist, None)
            mockft.list_files.assert_called_with('/remote/dname')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_challenge_data_package_file_found_match(self):
        temp_dir = tempfile.mkdtemp()
        try:
            dr = ExternalDataSubmissionFactory.DOCKEDRESULTS
            year = os.path.join(temp_dir, '2016')
            week = os.path.join(year, 'dataset.week.13')
            os.makedirs(week)
            params = D3RParameters()
            mockft = D3RParameters()
            mockft.list_files = Mock(return_value=['hi', 'celpp_week13_2015' +
                                                   dr + 'dname.tar.gz',
                                                   'celpp_week13_2016' + dr +
                                                   'yuck.tar.gz'])
            fac = ExternalDataSubmissionFactory(week, params)
            fac.set_file_transfer(mockft)
            dlist = fac._get_challenge_data_package_file('/remote', 'yuck')
            self.assertEqual(dlist,
                             '/remote/yuck/celpp_week13_2016' + dr +
                             'yuck.tar.gz')
            mockft.list_files.assert_called_with('/remote/yuck')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_external_data_submissions_raise_exception(self):
        params = D3RParameters()
        fac = ExternalDataSubmissionFactory('/foo', params)
        tlist = fac.get_external_data_submissions()
        self.assertEqual(len(tlist), 0)

    def test_get_external_data_submissions_none_subdir(self):
        params = D3RParameters()
        fac = ExternalDataSubmissionFactory('/foo', params)
        mockft = D3RParameters()
        mockft.list_dirs = Mock(return_value=None)
        fac.set_file_transfer(mockft)
        tlist = fac.get_external_data_submissions()
        self.assertEqual(len(tlist), 0)

    def test_get_external_data_submissions_no_dirs(self):
        params = D3RParameters()
        fac = ExternalDataSubmissionFactory('/foo', params)
        mockft = D3RParameters()
        mockft.list_dirs = Mock(return_value=[])
        fac.set_file_transfer(mockft)
        tlist = fac.get_external_data_submissions()
        self.assertEqual(len(tlist), 0)

    def test_get_external_data_submissions_no_chall_packages(self):
        temp_dir = tempfile.mkdtemp()
        try:
            year = os.path.join(temp_dir, '2016')
            week = os.path.join(year, 'dataset.week.13')
            os.makedirs(week)
            dr = ExternalDataSubmissionFactory.DOCKEDRESULTS
            params = D3RParameters()
            fac = ExternalDataSubmissionFactory(week, params)
            mockft = D3RParameters()
            mockft.connect = Mock()
            mockft.disconnect = Mock()
            mockft.get_remote_submission_dir = Mock(return_value='/remote')
            mockft.list_dirs = Mock(return_value=['yo'])
            mockft.list_files = Mock(return_value=['hi',
                                                   'celpp_week13_2015' + dr +
                                                   'dname.tar.gz',
                                                   'celpp_week13_2016' + dr +
                                                   'yuck.tar.gz'])
            fac.set_file_transfer(mockft)
            tlist = fac.get_external_data_submissions()
            self.assertEqual(len(tlist), 0)
            mockft.list_dirs.assert_called_with('/remote')
            mockft.list_files.assert_called_with('/remote/yo')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_external_data_submissions_one_extdatasubmission(self):
        temp_dir = tempfile.mkdtemp()
        try:
            year = os.path.join(temp_dir, '2017')
            week = os.path.join(year, 'dataset.week.13')
            os.makedirs(week)
            dr = ExternalDataSubmissionFactory.DOCKEDRESULTS
            params = D3RParameters()
            fac = ExternalDataSubmissionFactory(week, params)
            mockft = D3RParameters()
            mockft.connect = Mock()
            mockft.disconnect = Mock()
            mockft.get_remote_challenge_dir = Mock(return_value='/chall')
            mockft.delete_file = Mock(return_value=False)

            mockft.get_remote_submission_dir = Mock(return_value='/remote')
            mockft.list_dirs = Mock(return_value=['yo'])
            mockft.list_files = Mock(return_value=['hi',
                                                   'celpp_week13_2017' + dr +
                                                   'yo.tar.gz',
                                                   'celpp_week13_2016' + dr +
                                                   'yuck.tar.gz'])
            fac.set_file_transfer(mockft)
            tlist = fac.get_external_data_submissions()
            self.assertEqual(len(tlist), 1)
            self.assertEqual(tlist[0].get_name(), 'yo.extsubmission')
            self.assertEqual(tlist[0].get_remote_challenge_data_package(),
                             '/remote/yo/celpp_week13_2017' + dr + 'yo.tar.gz')

            mockft.list_dirs.assert_called_with('/remote')
            mockft.list_files.assert_called_with('/remote/yo')
            mockft.delete_file.assert_called_with('/chall/latest.txt')
        finally:
            shutil.rmtree(temp_dir)

    def test_get_external_data_submissions_two_extdatasubmission(self):
        temp_dir = tempfile.mkdtemp()
        try:
            year = os.path.join(temp_dir, '2017')
            week = os.path.join(year, 'dataset.week.13')
            os.makedirs(week)
            dr = ExternalDataSubmissionFactory.DOCKEDRESULTS
            params = D3RParameters()
            fac = ExternalDataSubmissionFactory(week, params)
            mockft = D3RParameters()
            mockft.connect = Mock()
            mockft.delete_file = Mock(return_value=True)
            mockft.get_remote_challenge_dir = Mock(return_value='/chall')
            mockft.get_remote_submission_dir = Mock(return_value='/remote')
            mockft.list_dirs = Mock(return_value=['yo', 'yuck'])
            mockft.list_files = Mock(return_value=['hi',
                                                   'celpp_week13_2017' + dr +
                                                   'yo.tar.gz',
                                                   'celpp_week13_2017' + dr +
                                                   'yuck.tar.gz'])
            fac.set_file_transfer(mockft)
            tlist = fac.get_external_data_submissions()
            self.assertEqual(len(tlist), 2)
            self.assertEqual(tlist[0].get_name(), 'yo.extsubmission')
            self.assertEqual(tlist[0].get_remote_challenge_data_package(),
                             '/remote/yo/celpp_week13_2017' + dr + 'yo.tar.gz')
            self.assertEqual(tlist[1].get_name(), 'yuck.extsubmission')
            self.assertEqual(tlist[1].get_remote_challenge_data_package(),
                             '/remote/yuck/celpp_week13_2017' + dr +
                             'yuck.tar.gz')
            mockft.list_dirs.assert_called_with('/remote')
            mockft.list_files.assert_called_with('/remote/yuck')
            mockft.delete_file.assert_called_with('/chall/latest.txt')
        finally:
            shutil.rmtree(temp_dir)

    def test_externaltask_get_set_remote_challenge_data_package(self):
        params = D3RParameters()
        task = ExternalDataSubmissionTask('/foo', 'name', '/remote', params)
        self.assertEqual(task.get_remote_challenge_data_package(), '/remote')
        task.set_remote_challenge_data_package('/blah')
        self.assertEqual(task.get_remote_challenge_data_package(), '/blah')

    def test_externaltask_can_run(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ExternalDataSubmissionTask(temp_dir, 'name', '/remote',
                                              params)
            self.assertTrue(task.can_run())

            task.create_dir()
            self.assertFalse(task.can_run())
            self.assertEqual(task.get_error(), 'stage.6.name.extsubmission '
                                               'already exists and status is '
                                               'unknown')
            open(os.path.join(task.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()

            self.assertFalse(task.can_run())
            self.assertEqual(task.get_error(), None)
        finally:
            shutil.rmtree(temp_dir)

    def test_is_tarmember_safe(self):
        params = D3RParameters()
        task = ExternalDataSubmissionTask('/foo', 'name',
                                          '/remote/celpp_week12_2016_'
                                          'dockedresults_name.tar.gz', params)
        # path starts with /
        val = task._is_tarmembername_safe('/celpp_week12', 'celpp_week12')
        self.assertFalse(val)
        self.assertEqual(task.get_email_log(), 'Skipping, path starts with '
                                               '/ : /celpp_week12\n')
        task.set_email_log('')
        # path does not start with chall_name
        val = task._is_tarmembername_safe('celpp', 'celpp_week12')
        self.assertFalse(val)
        self.assertEqual(task.get_email_log(), 'Skipping, path does not'
                                               ' conform: celpp\n')
        task.set_email_log('')

        # path has .. in it
        val = task._is_tarmembername_safe('celpp/../yo', 'celpp')
        self.assertFalse(val)
        self.assertEqual(task.get_email_log(), 'Skipping, found .. in path:'
                                               ' celpp/../yo\n')
        task.set_email_log('')
        chall_name = 'celpp_week12_2016_dockedresults_name'
        # valid path
        val = task._is_tarmembername_safe(chall_name + '/5xjv/yo.pdb',
                                          chall_name)
        self.assertTrue(val)
        self.assertEqual(task.get_email_log(), '')

    def test_untar_challenge_data_package_no_tarfile(self):

        params = D3RParameters()
        task = ExternalDataSubmissionTask('/foo', 'name',
                                          '/remote/hello.tar.gz', params)
        try:
            task._untar_challenge_data_package('hello')
            self.fail('Expected IOError')
        except IOError:
            pass

    def create_good_tarfile(self, temp_dir, nameprefix):
        tfiledir = os.path.join(temp_dir, nameprefix)
        tfile = tfiledir + '.tar.gz'

        os.makedirs(tfiledir)
        candidateone = '5juw'
        candidateonedir = os.path.join(tfiledir, candidateone)
        os.makedirs(candidateonedir)

        mypdb = 'apo-5juw_2eb2_docked.pdb'
        mypdbfile = os.path.join(candidateonedir, mypdb)
        f = open(mypdbfile, 'w')
        f.write('mypdb')
        f.flush()
        f.close()

        mymol = 'apo-5juw_2eb2_docked.mol'
        mymolfile = os.path.join(candidateonedir, mymol)
        f = open(mymolfile, 'w')
        f.write('mymol')
        f.flush()
        f.close()

        tar = tarfile.open(tfile, 'w:gz')
        tar.add(candidateonedir, arcname=nameprefix + '/' + candidateone)
        tar.close()
        shutil.rmtree(tfiledir)

        return tfile

    def create_bad_tarfile_dotdot_path(self, temp_dir, nameprefix):
        tfiledir = os.path.join(temp_dir, nameprefix)
        tfile = tfiledir + '.tar.gz'

        os.makedirs(tfiledir)
        candidateone = '5juw'
        candidateonedir = os.path.join(tfiledir, candidateone)
        os.makedirs(candidateonedir)

        mypdb = 'apo-5juw_2eb2_docked.pdb'
        mypdbfile = os.path.join(candidateonedir, mypdb)
        f = open(mypdbfile, 'w')
        f.write('mypdb')
        f.flush()
        f.close()

        mymol = 'apo-5juw_2eb2_..docked.mol'
        mymolfile = os.path.join(candidateonedir, mymol)
        f = open(mymolfile, 'w')
        f.write('mymol')
        f.flush()
        f.close()

        # add in a symlink for fun
        os.symlink(mymol, os.path.join(candidateonedir, 'hello'))

        tar = tarfile.open(tfile, 'w:gz')
        tar.add(candidateonedir, arcname='/' + nameprefix + '/' + candidateone)
        tar.close()
        shutil.rmtree(tfiledir)

        return tfile

    def test_untar_challenge_data_package_good_tarfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ExternalDataSubmissionTask(temp_dir, 'hello',
                                              'hi', params)
            task.create_dir()
            self.create_good_tarfile(task.get_dir(), 'hello')
            c = task._untar_challenge_data_package('hello.tar.gz')
            self.assertEqual(c, 'hello')
            self.assertEqual(task.get_email_log(), None)
            candidate = os.path.join(task.get_dir(), 'hello', '5juw')
            self.assertTrue(os.path.isdir(candidate))
            mypdb = os.path.join(candidate,
                                 'apo-5juw_2eb2_docked.pdb')
            mymol = os.path.join(candidate,
                                 'apo-5juw_2eb2_docked.mol')
            self.assertTrue(os.path.isfile(mypdb))
            self.assertTrue(os.path.isfile(mymol))
        finally:
            shutil.rmtree(temp_dir)

    def test_untar_challenge_data_package_dotdot_tarfile(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ExternalDataSubmissionTask(temp_dir, 'hello',
                                              'hi', params)
            task.create_dir()
            self.create_bad_tarfile_dotdot_path(task.get_dir(),
                                                'hello')
            c = task._untar_challenge_data_package('hello.tar.gz')
            self.assertEqual(c, 'hello')
            self.assertTrue('Skipping, found .. in '
                            'path: hello/5juw/apo-'
                            '5juw_2eb2_..docked.mol\n' in task.get_email_log())

            self.assertTrue('Ignoring non dir/file '
                            'entry in tar 5juw/'
                            'hello\n' in task.get_email_log())

            candidate = os.path.join(task.get_dir(), 'hello', '5juw')
            self.assertTrue(os.path.isdir(candidate))
            mypdb = os.path.join(candidate,
                                 'apo-5juw_2eb2_docked.pdb')
            mymol = os.path.join(candidate,
                                 'apo-5juw_2eb2.._docked.mol')
            self.assertTrue(os.path.isfile(mypdb))
            self.assertFalse(os.path.isfile(mymol))
        finally:
            shutil.rmtree(temp_dir)

    def test_untar_challenge_data_package_wrong_prefix(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              'hi', params)
            task.create_dir()
            tfile = self.create_good_tarfile(task.get_dir(),
                                             'yo')
            shutil.move(tfile, os.path.join(task.get_dir(),
                                            'byte.tar.gz'))
            c = task._untar_challenge_data_package('byte.tar.gz')
            self.assertEqual(c, 'byte')

            candidate = os.path.join(task.get_dir(), 'hello', '5juw')
            self.assertFalse(os.path.isdir(candidate))
            mypdb = os.path.join(candidate,
                                 'apo-5juw_2eb2_docked.pdb')
            mymol = os.path.join(candidate,
                                 'apo-5juw_2eb2.._docked.mol')
            self.assertFalse(os.path.isfile(mypdb))
            self.assertFalse(os.path.isfile(mymol))
        finally:
            shutil.rmtree(temp_dir)

    def test_move_challenge_data_package_into_task_dir_raises_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              'hi', params)
            task.create_dir()
            try:
                task._move_challenge_data_package_into_task_dir('foo')
                self.fail('expected IOError')
            except OSError:
                pass
        finally:
            shutil.rmtree(temp_dir)

    def test_move_challenge_data_package_into_task_dir_valid(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              'hi', params)
            task.create_dir()
            basedir = os.path.join(task.get_dir(), 'foo')
            os.makedirs(basedir)
            can1 = os.path.join(basedir, '5juw')
            os.makedirs(can1)
            pdb1 = os.path.join(can1, 'apo.pdb')
            open(pdb1, 'a').close()
            mol1 = os.path.join(can1, 'apo.mol')
            open(mol1, 'a').close()

            can2 = os.path.join(basedir, 'abcd')
            os.makedirs(can2)
            pdb2 = os.path.join(can2, 'apo.pdb')
            open(pdb2, 'a').close()
            mol2 = os.path.join(can2, 'apo.mol')
            open(mol2, 'a').close()

            task._move_challenge_data_package_into_task_dir('foo')
            can3 = os.path.join(task.get_dir(), '5juw')
            can4 = os.path.join(task.get_dir(), 'abcd')
            self.assertTrue(os.path.isdir(can3))
            self.assertTrue(os.path.isdir(can4))
            self.assertTrue(os.path.isfile(os.path.join(can3, 'apo.pdb')))
            self.assertTrue(os.path.isfile(os.path.join(can3, 'apo.mol')))
            self.assertTrue(os.path.isfile(os.path.join(can4, 'apo.pdb')))
            self.assertTrue(os.path.isfile(os.path.join(can4, 'apo.mol')))

        finally:
            shutil.rmtree(temp_dir)

    def test_get_summary_of_docked_results(self):
        params = D3RParameters()
        task = ExternalDataSubmissionTask('/foo', 'yo',
                                          'hi', params)
        self.assertEqual(task._get_summary_of_docked_results(), '')

    def test_download_remote_challenge_data_package_with_retry_retry_neg(self):
        params = D3RParameters()
        task = ExternalDataSubmissionTask('/foo', 'yo',
                                          'hi', params)
        task.set_download_max_retry_count(-1)
        task.set_download_retry_sleep(0)
        try:
            task._download_remote_challenge_data_package_with_retry()
            self.fail('Expected ChallengePackageDownloadError')
        except ChallengePackageDownloadError as e:
            self.assertEqual(str(e), 'Unable to download hi')

    def test_download_remote_challenge_data_package_with_retry_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            pkg = '/remote/celpp1_2017_dockedresults_yo.tar.gz'
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              pkg, params)
            task.set_download_retry_sleep(0)
            task.set_download_max_retry_count(1)
            mockft = D3RParameters()
            mockft.connect = Mock(side_effect=IOError('error'))
            mockft.download_file = Mock(return_value=True)
            mockft.disconnect = Mock(return_value=None)
            task.set_file_transfer(mockft)
            try:
                task._download_remote_challenge_data_package_with_retry()
            except ChallengePackageDownloadError as e:
                self.assertEqual(str(e), 'Unable to download ' +
                                 'celpp1_2017_dockedresults_yo.tar.gz')
        finally:
            shutil.rmtree(temp_dir)

    def test_download_remote_challenge_data_package_with_retry_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            pkg = '/remote/celpp1_2017_dockedresults_yo.tar.gz'
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              pkg, params)
            task.set_download_retry_sleep(0)
            task.set_download_max_retry_count(1)
            mockft = D3RParameters()
            mockft.connect = Mock()
            mockft.download_file = Mock(return_value=False)
            mockft.disconnect = Mock(return_value=None)
            task.set_file_transfer(mockft)
            try:
                task._download_remote_challenge_data_package_with_retry()
            except ChallengePackageDownloadError as e:
                self.assertEqual(str(e), 'Unable to download ' +
                                 'celpp1_2017_dockedresults_yo.tar.gz')
        finally:
            shutil.rmtree(temp_dir)

    def test_download_remote_challenge_data_package_with_1st_retry_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            pkg = '/remote/celpp1_2017_dockedresults_yo.tar.gz'
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              pkg, params)
            task.set_download_retry_sleep(0)
            task.set_download_max_retry_count(3)
            mockft = D3RParameters()
            mockft.connect = Mock()
            mockft.download_file = Mock()
            mockft.download_file.side_effect = [False, True]
            mockft.disconnect = Mock(return_value=None)
            task.set_file_transfer(mockft)
            c = task._download_remote_challenge_data_package_with_retry()
            self.assertEqual(c, 'celpp1_2017_dockedresults_yo.tar.gz')
        finally:
            shutil.rmtree(temp_dir)

    def test__download_remote_challenge_data_package_raise_exception(self):
        params = D3RParameters()
        task = ExternalDataSubmissionTask('/foo', 'yo',
                                          'hi', params)
        try:
            task._download_remote_challenge_data_package('foo')
            self.fail('Expected attribute error cause file transfer'
                      'object is not set')
        except AttributeError:

            pass

    def test__download_remote_challenge_data_package_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            pkg = '/remote/celpp2_3.tar.gz'
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              pkg, params)
            task.create_dir()
            mockft = D3RParameters()
            mockft.connect = Mock(return_value=None)
            mockft.download_file = Mock(return_value=True)
            mockft.disconnect = Mock(return_value=None)
            task.set_file_transfer(mockft)
            val = task._download_remote_challenge_data_package('celpp2_3.'
                                                               'tar.gz')
            self.assertTrue(val)
            localfile = os.path.join(task.get_dir(), 'celpp2_3.tar.gz')
            mockft.download_file.assert_called_with(pkg, localfile)
            mockft.connect.assert_called_with()
            mockft.disconnect.assert_called_with()
        finally:
            shutil.rmtree(temp_dir)

    def test_run_unable_to_run_cause_can_run_is_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            pkg = '/remote/celpp2_3.tar.gz'
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              pkg, params)
            task.create_dir()
            task.run()
            self.assertEqual(task.get_email_log(), None)
            self.assertEqual(task.get_error(), 'stage.6.yo.extsubmission '
                                               'already exists and status '
                                               'is unknown')
        finally:
            shutil.rmtree(temp_dir)

    def test_runtask_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            pkg = '/remote/celpp1_2017_dockedresults_yo.tar.gz'
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              pkg, params)
            mockft = D3RParameters()
            mockft.connect = Mock(return_value=None)
            mockft.download_file = Mock(return_value=True)
            mockft.disconnect = Mock(return_value=None)
            task.set_file_transfer(mockft)
            task.create_dir()
            self.create_good_tarfile(task.get_dir(),
                                     'celpp1_2017_dockedresults_yo')
            task._runtask()
            self.assertEqual(task.get_error(), None)
            self.assertEqual(task.get_email_log(), '')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fail(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            pkg = '/remote/celpp1_2017_dockedresults_yo.tar.gz'
            task = ExternalDataSubmissionTask(temp_dir, 'yo',
                                              pkg, params)
            mockft = D3RParameters()
            mockft.connect = Mock(side_effect=IOError('error'))
            mockft.download_file = Mock(return_value=True)
            mockft.disconnect = Mock(return_value=None)
            task.set_file_transfer(mockft)
            task.run()
            self.assertEqual(task.get_error(), 'Caught exception Unable to '
                                               'download celpp1_2017_docked'
                                               'results_yo.tar.gz')
            self.assertEqual(task.get_email_log(), 'Try # 1 of 3 to download '
                                                   'celpp1_2017_dockedresults'
                                                   '_yo.tar.gz\nTry # 2 of 3 '
                                                   'to download celpp1_2017_d'
                                                   'ockedresults_yo.tar.gz\n'
                                                   'Try # 3 of 3 to download '
                                                   'celpp1_2017_dockedresults'
                                                   '_yo.tar.gz\n')

        finally:
            shutil.rmtree(temp_dir)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    "argparse",
    "lockfile",
    "psutil",
    "biopython",
    "xlsxwriter",
    "ftpretty",
    "python-dateutil",
    "easywebdav"
]

test_requirements = [
    "argparse",
    "lockfile",
    "psutil",
    "biopython",
    "xlsxwriter",
    "ftpretty",
    "python-dateutil",
    "mock",
    "easywebdav"
]

setup(
    name='d3r',
    version='1.7.1',
    description='Drug Design Data Resource CELPP Runner is an application to run the filtering, docking, and '
                'evaluation of new sequences from wwpdb',
    long_description=readme + '\n\n' + history,
    author='Christopher Churas <churas@ncmir.ucsd.edu>,'
           'Shaui Liu <shuailiu25@gmail.com>,'
           'Jeff Wagner <j5wagner@ucsd.edu>,'
           'Rob Swift <rvswift@ucsd.edu>',
    author_email='drugdesigndata@gmail.com',
    url='https://github.com/drugdata/D3R',
    packages=[
        'd3r', 'd3r.blast', 'd3r.filter', 'd3r.utilities', 'd3r.celpp', 
        'd3r.celppade'
    ],
    package_dir={'d3r':
                 'd3r'},
    include_package_data=True,
    install_requires=requirements,
    license="Other",
    zip_safe=False,
    keywords='d3r',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Science/Research',
        'License :: Free for non-commercial use',
        'Natural Language :: English',
        'Environment :: Console',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering :: Chemistry'
    ],
    scripts = ['d3r/celpprunner.py', 'd3r/blastnfilter.py',
               'd3r/postanalysis.py', 'd3r/proteinligprep.py',
               'd3r/glidedocking.py', 'd3r/evaluate.py',
               'd3r/celppreports.py', 'd3r/vinadocking.py',
               'd3r/genchallengedata.py','d3r/chimera_proteinligprep.py',
               'd3r/getchallengedata.py','d3r/packdockingresults.py',
               'd3r/post_evaluation.py'
               ],
    test_suite='tests',
    tests_require=test_requirements
)



__author__ = 'robswift'

import os

class Writer(object):
    """
    Base writer class
    """
    def __init__(self, out_dir):
        self.query = None
        self.out_dir = out_dir
        self.handle = None

    def write_query_header(self):
        out = []
        out.append('Query ID: {id}|Sequence count: {seq_count}|Dock count: {dock_count}'.format(
            id=self.query.pdb_id, seq_count=self.query.sequence_count, dock_count=self.query.dock_count))
        for l in out: self.handle.write("%s\n" % l)

    def write_query_status(self):
        """
        Check set and write the status of a query.
        """
        out = []
        if self.query.triage:
            query_status = 'Discarded'
        else:
            query_status = 'Retained'
        out.append('Status: {status}'.format(status=query_status))
        if query_status == 'Discarded':
            for reason in self.query.reasons_to_triage:
                out.append('Reason: {reason}'.format(reason=reason))
        for l in out: self.handle.write("%s\n" % l)

    def write_query_chains(self):
        out = []
        for seq in self.query.sequences:
            out.append('Chain_{id}: Length = {length}'.format(id=seq.id.split('_')[1], length=len(seq)))
        for l in out: self.handle.write("%s\n" % l)

    def write_query_ligands(self):
        if self.query.dock_count == 0:
            pass
        else:
            out = []
            for lig in self.query.dock:
                out.append('Ligand: {res}'.format(res=lig.resname))
            for l in out: self.handle.write("%s\n" % l)

    def write_query_ph(self):
        out = []
        if self.query.exp_ph:
            out.append('Crystallization pH: {ph}'.format(ph=self.query.exp_ph))
        else:
            out.append('Crystallization pH: N/A')
        for l in out: self.handle.write("%s\n" % l)

    def write_hit_header(self, hit):
        out = []
        out.append('  Hit ID: {id}|Sequence count: {seq_count}|Chain count: {chain_count}|Dock count:'
                   ' {dock_count}'.format(id=hit.pdb_id, seq_count=hit.sequence_count, chain_count=hit.chain_count,
                                          dock_count=hit.dock_count))
        for l in out: self.handle.write("%s\n" % l)

    def write_hit_status(self, hit):
        out = []
        if hit.triage:
            hit_status = 'Discarded'
        elif hit.retain:
            hit_status = 'Elected for docking'
        else:
            hit_status = 'Docking candidate'
        out.append('  Status: {stat}'.format(stat=hit_status))
        if hit_status == 'Discarded':
            for reason in hit.reasons_to_triage:
                out.append('  Reason: {reason}'.format(reason=reason))
        elif hit_status == 'Elected for docking':
            for reason in hit.reasons_to_retain:
                out.append('  Reason: {reason}'.format(reason=reason))
        for l in out: self.handle.write("%s\n" % l)

    def write_hit_chains(self, hit):
        out = []
        hit.sequences.sort(key=lambda seq: seq.blast_hit, reverse=True)
        for seq in hit.sequences:
            if seq.blast_hit:
                for qa in seq.query_alignments:
                    out.append('  Chain_{id}_{no}:  Length = {length}|%identity = {identity:.2} '
                               '(with Chain_{query_id})|%coverage = {coverage:.2} '
                               '(with Chain_{query_id})'.format(id=seq.hit_chain_id, no=seq.hit_sequence_id,
                                                                length=len(seq.seq_record), identity=qa.identity,
                                                                coverage=qa.coverage, query_id=qa.query_chain_id))
            else:
                out.append('  Chain_{id}_{no}: Length = {length}|Not a BLAST hit'.format(id=seq.hit_chain_id,
                                                                                          no=seq.hit_sequence_id,
                                                                                          length=len(seq.seq_record)))

        for l in out: self.handle.write("%s\n" % l)

    def write_hit_ligands(self, hit):
        if hit.dock_count == 0:
            pass
        else:
            out = []
            for lig in hit.dock:
                line = []
                line.append('  Ligand: {res}'.format(res=lig.resname))
                for mcss in lig.mcsss:
                    line.append('|MCSS size: {s} (with {l})'.format(s=mcss.size, l=mcss.reference))
                out.append(''.join(line))
            for l in out: self.handle.write("%s\n" % l)

    def write_hit_method(self, hit):
        out = []
        out.append('  Method: {method}|Resolution: {res}'.format(method=hit.exp_method, res=hit.resolution))
        for l in out: self.handle.write("%s\n" % l)

    def write_blank_lines(self, number):
        out = []
        for i in range(number):
            out.append('')
        for l in out: self.handle.write("%s\n" % l)

    def write_hit(self, hit):
        self.write_hit_header(hit)
        self.write_hit_status(hit)
        self.write_hit_chains(hit)
        self.write_hit_ligands(hit)
        self.write_hit_method(hit)
        self.write_blank_lines(1)


class WriteText(object):
    """
    This is the new txt writer that meets the format that Shuai wanted.
    """
    def __init__(self, out_dir):
        self.query = None
        self.out_dir = out_dir
        self.handle = None
        self.most_similar = []
        self.least_similar = []
        self.highest_res = []
        self.apo = []
        self.highest_tanimoto = []
        self.reasons = {
            1: 'The BLAST hit is bound to the ligand with the largest maximum common substructure',
            2: 'The BLAST hit is bound to the ligand with the smallest maximum common substructure',
            3: 'The BLAST hit is the highest resolution holo structure',
            4: 'The BLAST hit is the highest resolution apo structure',
            5: 'The BLAST hit is bound to the ligand with the highest tanimoto score in structural similarity',
        }

    def write_txt(self, query):
        self.reinitialize(query)
        self.categorize()
        self.open_file()
        self.write_query()
        self.write_hits()
        self.close_file()

    def write_query(self):
        self.write_query_header()
        self.write_query_ph()
        self.write_query_ligands()

    def write_query_header(self):
        out = []
        out.append('query, {id}'.format(id=self.query.pdb_id))
        for l in out: self.handle.write("%s\n" % l)

    def write_query_ph(self):
        out=[]
        if self.query.exp_ph:
            out.append('ph, {ph}'.format(ph=self.query.exp_ph))
        else:
            out.append('ph, N/A')
        for l in out: self.handle.write("%s\n" % l)

    def write_query_ligands(self):
        if self.query.dock_count == 0:
            pass
        else:
            out = []
            for lig in self.query.dock:
                out.append('ligand, {res}'.format(res=lig.resname))
                out.append('inchi, {inchi}'.format(inchi=lig.inchi))
                #modified by sliu 08/08 
                out.append('size, {size}'.format(size=lig.size))
                out.append('rotatable_bond, {rot}'.format(rot=lig.rot))
                #out.append('heavy size, {heavy_size}'.format(heavy_size=lig.heavy))
            for l in out: self.handle.write("%s\n" % l)

    def write_hits(self):
        #here change to only write out the top 10 largest
        for hit in self.most_similar[:10]:
            self.write_largest(hit)
        for hit in self.least_similar:
            self.write_smallest(hit)
        #sliu 01092017 change to top ten in holo structures
        for hit in self.highest_res[:10]:
            self.write_holo(hit)
        for hit in self.apo:
            self.write_apo(hit)
        for hit in self.highest_tanimoto[:10]:
            self.write_highest_tanimoto(hit)

    def write_largest(self, hit):
        out = []
        #only write out the dockable mcss in the first ordered chain
        largest_lig = hit.dock[hit.largest_index[0]]
        for mcss in largest_lig.mcsss:
            out.append("LMCSS, {pdb_id}, {lig}, chain: {chain_id}, (size: {lig_size}, mcss_size: {mcss_size}, resolution: {resolution}) ".format(pdb_id=hit.pdb_id, lig=mcss.test, chain_id=hit.largest_mcss_chain[0], lig_size= largest_lig.size, mcss_size= mcss.size, resolution= "%4.4s"%hit.resolution ))
        for l in out: self.handle.write("%s\n" % l)

    def write_smallest(self, hit):
        out = []
        smallest_lig = hit.dock[hit.smallest_index[0]]
        for mcss in hit.dock[hit.smallest_index[0]].mcsss:
            #out.append("SMCSS, {pdb_id}, {lig} ".format(pdb_id=hit.pdb_id, lig=mcss.test ))
            out.append("SMCSS, {pdb_id}, {lig}, chain: {chain_id}, (size: {lig_size}, mcss_size: {mcss_size}, resolution: {resolution}) ".format(pdb_id=hit.pdb_id, lig=mcss.test, chain_id=hit.smallest_mcss_chain[0], lig_size= smallest_lig.size, mcss_size= mcss.size, resolution= "%4.4s"%hit.resolution ))
        for l in out: self.handle.write("%s\n" % l)

    def write_holo(self, hit):
        out = []
        largest_lig = hit.dock[hit.largest_index[0]] 
        for mcss in largest_lig.mcsss:
            out.append("hiResHolo, {pdb_id}, {lig}, chain: {chain_id}, (resolution: {resolution})".format(pdb_id=hit.pdb_id, lig=mcss.test, chain_id=hit.largest_mcss_chain[0], resolution= "%4.4s"%hit.resolution))
#        for lig in hit.dock[:1]:
#            for mcss in lig.mcsss:
#                out.append("hiResHolo, {pdb_id}, {lig}".format(pdb_id=hit.pdb_id, lig=mcss.test))
        for l in out: self.handle.write("%s\n" % l)

    def write_apo(self, hit):
        out = []
        out.append("hiResApo, {pdb_id}".format(pdb_id=hit.pdb_id))
        for l in out: self.handle.write("%s\n" %l)
    def write_highest_tanimoto(self, hit):
        out = []
        #only write out the dockable mcss in the first ordered chain
        largest_lig = hit.dock[hit.highest_tanimoto_index[0]]
        for mcss in largest_lig.mcsss:
            out.append("hiTanimoto, {pdb_id}, {lig}, chain: {chain_id}, (tanimoto_similarity: {tanimoto}, resolution: {resolution})".format(pdb_id=hit.pdb_id, lig=mcss.test, chain_id=hit.highest_tanimoto_chain[0], tanimoto= "%4.4s"%(mcss.tanimoto), resolution= "%4.4s"%hit.resolution  ))
        for l in out: self.handle.write("%s\n" % l)

    def reinitialize(self, query):
        self.most_similar = []
        self.least_similar = []
        self.highest_res = []
        self.apo = []
        self.query = query

    def open_file(self):
        f = os.path.join(self.out_dir, '{stem}.txt'.format(stem=self.query.pdb_id))
        self.handle = open(f, 'w')

    def close_file(self):
        self.handle.close()

    def categorize(self):
        for hit in self.query.hits:
            if hit.retain:
                for reason in hit.reasons_to_retain:
                    if reason == self.reasons[1] and hit.pdb_id:
                        self.most_similar.append(hit)
                    if reason == self.reasons[2] and hit.pdb_id:
                        self.least_similar.append(hit)
                    if reason == self.reasons[3] and hit.pdb_id:
                        self.highest_res.append(hit)
                    if reason == self.reasons[4] and hit.pdb_id:
                        self.apo.append(hit)
                    #add tanimoto 09/01 sliu
                    if reason == self.reasons[5] and hit.pdb_id:
                        self.highest_tanimoto.append(hit)
        #here rank the list by the resolution
        self.most_similar.sort(key=lambda hit:(float(hit.resolution)))
        self.least_similar.sort(key=lambda hit:(float(hit.resolution)))
        self.highest_res.sort(key=lambda hit:(float(hit.resolution)))
        self.apo.sort(key=lambda hit:(float(hit.resolution)))
        self.highest_tanimoto.sort(key=lambda hit:(float(hit.resolution)))


# class WriteTxt(Writer):
#    """
#    Writes a txt file that describes a query and the corresponding blast hits elected for docking. This is the old
#    class that is less useful for Shuai.
#    """
#    def __init__(self, *args, **kwargs):
#        super(WriteTxt, self).__init__(*args, **kwargs)
#        self.most_similar = []
#        self.least_similar = []
#        self.highest_res = []
#        self.apo = []
#        self.reasons = {
#            1: 'The BLAST hit is bound to the ligand with the largest maximum common substructure',
#            2: 'The BLAST hit is bound to the ligand with the smallest maximum common substructure',
#            3: 'The BLAST hit is the highest resolution structure',
#            4: 'The BLAST hit is the highest resolution apo structure',
#        }
#
#    def write_txt(self, query):
#        self.reinitialize(query)
#        self.categorize()
#        self.open_file()
#        self.write_query()
#        self.write_hits()
#        self.close_file()
#
#    def write_query(self):
#        self.write_query_header()
#        self.write_query_chains()
#        self.write_query_ligands()
#        self.write_blank_lines(1)
#
#    def write_hits(self):
#        for hit in self.most_similar:
#            self.write_hit(hit)
#        for hit in self.least_similar:
#            self.write_hit(hit)
#        for hit in self.highest_res:
#            self.write_hit(hit)
#        for hit in self.apo:
#            self.write_hit(hit)
#
#    def reinitialize(self, query):
#        self.most_similar = []
#        self.least_similar = []
#        self.highest_res = []
#        self.apo = []
#        self.query = query
#
#    def open_file(self):
#        f = os.path.join(self.out_dir, '{stem}.txt'.format(stem=self.query.pdb_id))
#        self.handle = open(f, 'w')
#
#    def close_file(self):
#        self.handle.close()
#
#    def categorize(self):
#        assigned = []
#        for hit in self.query.hits:
#            if hit.retain:
#                for reason in hit.reasons_to_retain:
#                    if reason == self.reasons[1] and hit.pdb_id not in assigned:
#                        self.most_similar.append(hit)
#                        assigned.append(hit.pdb_id)
#                    if reason == self.reasons[2] and hit.pdb_id not in assigned:
#                        self.least_similar.append(hit)
#                        assigned.append(hit.pdb_id)
#                    if reason == self.reasons[3] and hit.pdb_id not in assigned:
#                        self.highest_res.append(hit)
#                        assigned.append(hit.pdb_id)
#                    if reason == self.reasons[4] and hit.pdb_id not in assigned:
#                        self.apo.append(hit)
#                        assigned.append(hit.pdb_id)

class WriteLog(Writer):
    """
    Writes a log file that describes a query and the corresponding blst hits
    """
    def __init__(self, *args, **kwargs):
        super(WriteLog, self).__init__(*args, **kwargs)
        self.open_file()

    def open_file(self):
        f = os.path.join(self.out_dir, 'blastnfilter.log')
        self.handle = open(f, 'a')

    def close_file(self):
        self.handle.close()

    def write_log(self, query):
        self.query = query
        self.write_query()
        self.write_hits()

    def write_query(self):
        self.write_query_header()
        self.write_query_status()
        self.write_query_chains()
        self.write_query_ligands()
        self.write_query_ph()
        self.write_blank_lines(1)

    def write_hits(self):
        for hit in self.query.hits:
            self.write_hit(hit)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import unittest
import tempfile
import shutil
import os.path
from mock import Mock

from d3r.celpp.filetransfer import WebDavFileTransfer

"""
test_uploader
----------------------------------

Tests for `uploader` module.
"""


class MockFtp(object):
    """Dummy ftp connection
    """


class TestFtpFileTransfer(unittest.TestCase):

    def setUp(self):
        pass

    def test_constructor(self):
        # test passing None which means we set everything manually
        foo = WebDavFileTransfer(None)

        self.assertEqual(foo.get_host(), None)
        self.assertEqual(foo.get_password(), None)
        self.assertEqual(foo.get_user(), None)
        self.assertEqual(foo.get_remote_dir(), '')
        self.assertEqual(foo.get_error_msg(), None)
        self.assertEqual(foo.get_remote_challenge_dir(), None)

        temp_dir = tempfile.mkdtemp()
        try:
            # test passing non existant config file
            try:
                foo = WebDavFileTransfer(os.path.join(temp_dir,
                                                      'doesnotexist'))
                self.fail('Expected IOError')
            except IOError:
                pass

            # test passing empty config file
            open(os.path.join(temp_dir, 'empty'), 'a').close()
            foo = WebDavFileTransfer(os.path.join(temp_dir, 'empty'))
            self.assertEqual(foo.get_host(), None)
            self.assertEqual(foo.get_password(), None)
            self.assertEqual(foo.get_user(), None)
            self.assertEqual(foo.get_remote_dir(), '')
            self.assertEqual(foo.get_remote_challenge_dir(), None)

            # test getters and setters
            foo.set_connect_timeout(10)
            foo.set_connection('hi')
            foo.set_host('host')
            foo.set_password('pass')
            foo.set_remote_dir('/remote')
            foo.set_user('user')
            foo.set_remote_challenge_dir('/chall')

            self.assertEqual(foo.get_connect_timeout(), 10)
            self.assertEqual(foo.get_host(), 'host')
            self.assertEqual(foo.get_password(), 'pass')
            self.assertEqual(foo.get_remote_dir(), '/remote')
            self.assertEqual(foo.get_user(), 'user')
            self.assertEqual(foo.get_remote_challenge_dir(), '/chall')
        finally:
            shutil.rmtree(temp_dir)

    def test_connect(self):
        # test where alt_ftp_con is None and we use easywebdav
        foo = WebDavFileTransfer(None)
        foo.set_host('doesnotexist')
        foo.set_user('user')
        foo.set_password('')
        foo.set_remote_dir('/remote')
        foo.set_connect_timeout(0)
        self.assertTrue(foo.connect())

        # test where alt_ftp_con is set
        foo = WebDavFileTransfer(None)
        foo.set_connection('hi')
        foo.connect()
        self.assertEqual(foo._ftp, 'hi')

    def test_disconnect(self):
        # test disconnect where _ftp is None
        foo = WebDavFileTransfer(None)
        foo.disconnect()

        # test disconnect where _ftp is set and
        # so is _alt_ftp_con
        foo = WebDavFileTransfer(None)
        foo.set_connection('hi')
        foo.connect()
        foo.disconnect()

        # test disconnect where _ftp is set and
        # _alt_ftp_con is None
        foo = WebDavFileTransfer(None)
        foo.connect()
        foo._alt_ftp_con = None
        foo.disconnect()

    def test_upload_file_direct_none_passed_as_file(self):
        foo = WebDavFileTransfer(None)

        self.assertEqual(foo.upload_file_direct(None, '/remote_dir',
                                                'remote_filename'), False)

        self.assertEqual(foo.get_error_msg(), 'File passed in is None')

    def test_upload_file_direct_file_is_not_a_file(self):
        temp_dir = tempfile.mkdtemp()
        try:
            foo = WebDavFileTransfer(None)
            noexist = os.path.join(temp_dir, 'noexist')
            self.assertEqual(foo.upload_file_direct(noexist,
                                                    '/remote_dir',
                                                    'remote_filename'), False)

            self.assertEqual(foo.get_error_msg(), noexist +
                             ' is not a file')
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_direct_remote_dir_is_none(self):
        temp_dir = tempfile.mkdtemp()
        try:
            foo = WebDavFileTransfer(None)
            afile = os.path.join(temp_dir, 'afile')
            open(afile, 'a').close()
            self.assertEqual(foo.upload_file_direct(afile, None,
                                                    'remote_filename'), False)
            self.assertEqual(foo.get_error_msg(), 'remote_dir is None')
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_direct_remote_file_name_is_none(self):
        temp_dir = tempfile.mkdtemp()
        try:
            foo = WebDavFileTransfer(None)
            afile = os.path.join(temp_dir, 'afile')
            open(afile, 'a').close()
            self.assertEqual(foo.upload_file_direct(afile, '/foo', None),
                             False)
            self.assertEqual(foo.get_error_msg(), 'remote_file_name is None')
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_direct_put_throws_exception(self):
        temp_dir = tempfile.mkdtemp()
        try:
            mockftp = MockFtp()
            mockftp.upload = Mock(side_effect=IOError('hi'))
            foo = WebDavFileTransfer(None)
            foo.set_connection(mockftp)
            foo.connect()
            afile = os.path.join(temp_dir, 'afile')
            open(afile, 'a').close()
            self.assertEqual(foo.upload_file_direct(afile, '/foo', 'name'),
                             False)
            self.assertEqual(foo.get_error_msg(), 'Unable to upload ' + afile +
                             ' to /foo/name : hi')
            foo.disconnect()
        finally:
            shutil.rmtree(temp_dir)

    def test_upload_file_direct_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            mockftp = MockFtp()
            mockftp.upload = Mock()
            foo = WebDavFileTransfer(None)
            foo.set_connection(mockftp)
            foo.connect()
            fakefile = os.path.join(temp_dir, 'foo')
            f = open(fakefile, 'w')
            f.write('123')
            f.flush()
            f.close()
            self.assertEqual(foo.upload_file_direct(fakefile, '/foo', 'name'),
                             True)
            self.assertEqual(foo.get_error_msg(), None)
            self.assertEqual(foo.get_upload_summary(), '1 (3 bytes) files '
                                                       'uploaded in 0 seconds'
                                                       ' to host Unset:')
            foo.disconnect()
        finally:
            shutil.rmtree(temp_dir)

    def test_get_upload_summary_remotedir_not_set(self):
        foo = WebDavFileTransfer(None)
        sum = foo.get_upload_summary()
        self.assertEqual(sum, '0 (0 bytes) files uploaded in 0 seconds to'
                              ' host Unset:')

    def test_download_file_remote_file_params_none(self):
        foo = WebDavFileTransfer(None)
        self.assertFalse(foo.download_file(None, None))
        self.assertEqual(foo.get_error_msg(), 'remote_file None')

        self.assertFalse(foo.download_file('/hi', None))
        self.assertEqual(foo.get_error_msg(), 'local_file None')

    def test_download_file_not_connected(self):
        foo = WebDavFileTransfer(None)
        self.assertFalse(foo.download_file('/remote/bye', '/local/hi'))
        self.assertEqual(foo.get_error_msg(), "Unable to download /remote/bye "
                                              "to /local/hi : 'NoneType' "
                                              "object has no attribute "
                                              "'download'")

    def test_download_file_success(self):
        mockftp = MockFtp()
        mockftp.download = Mock()
        foo = WebDavFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        self.assertTrue(foo.download_file('/a/b', '/c/d'))
        self.assertEqual(foo.get_error_msg(), None)
        foo.disconnect()
        mockftp.download.assert_called_with('/a/b', '/c/d')

    def test_download_file_fail(self):
        mockftp = MockFtp()
        mockftp.download = Mock(side_effect=IOError('error'))
        foo = WebDavFileTransfer(None)
        foo.set_connection(mockftp)
        foo.connect()
        self.assertFalse(foo.download_file('/a/b', '/c/d'))
        self.assertEqual(foo.get_error_msg(), 'Unable to download /a/b to '
                                              '/c/d : error')
        foo.disconnect()
        mockftp.download.assert_called_with('/a/b', '/c/d')

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



__author__ = 'churas'

import unittest
import tempfile
import os.path

"""
test_vina
--------------------------------

Tests for `vina` module.
"""

import shutil
import os
from d3r.celpp.task import D3RParameters
from d3r.celpp.task import D3RTask
from d3r.celpp.vina import AutoDockVinaTask
from d3r.celpp.chimeraprep import ChimeraProteinLigPrepTask


class TestAutoDockVinaTask(unittest.TestCase):
    def setUp(self):
        pass

    def test_get_uploadable_files(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            task = AutoDockVinaTask(temp_dir, params)
            # try with no dir
            self.assertEqual(task.get_uploadable_files(), [])

            # try with empty dir
            task.create_dir()
            self.assertEqual(task.get_uploadable_files(), [])

            # try with stderr/out files
            errfile = os.path.join(task.get_dir(), 'vinadocking.py.stderr')
            open(errfile, 'a').close()
            outfile = os.path.join(task.get_dir(), 'vinadocking.py.stdout')
            open(outfile, 'a').close()
            flist = task.get_uploadable_files()
            self.assertEqual(len(flist), 2)
            flist.index(errfile)
            flist.index(outfile)

        finally:
            shutil.rmtree(temp_dir)

    def test_can_run(self):
        temp_dir = tempfile.mkdtemp()
        try:
            # no proteinligprep task found so it cannot run
            params = D3RParameters()
            vina = AutoDockVinaTask(temp_dir, params)
            self.assertEqual(vina.can_run(), False)
            self.assertEqual(vina.get_error(),
                             'chimeraprep task has notfound status')

            # proteinligprep filter running
            proteinligprep = ChimeraProteinLigPrepTask(temp_dir, params)
            proteinligprep.create_dir()
            open(os.path.join(proteinligprep.get_dir(), D3RTask.START_FILE),
                 'a').close()
            vina = AutoDockVinaTask(temp_dir, params)
            self.assertEqual(vina.can_run(), False)
            self.assertEqual(vina.get_error(),
                             'chimeraprep task has start status')

            # proteinligprep failed
            error_file = os.path.join(proteinligprep.get_dir(),
                                      D3RTask.ERROR_FILE)
            open(error_file, 'a').close()
            vina = AutoDockVinaTask(temp_dir, params)
            self.assertEqual(vina.can_run(), False)
            self.assertEqual(vina.get_error(),
                             'chimeraprep task has error status')

            # proteinligprep success
            os.remove(error_file)
            open(os.path.join(proteinligprep.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            vina = AutoDockVinaTask(temp_dir, params)
            self.assertEqual(vina.can_run(), True)
            self.assertEqual(vina.get_error(), None)

            # vina task exists already
            vina = AutoDockVinaTask(temp_dir, params)
            vina.create_dir()
            self.assertEqual(vina.can_run(), False)
            self.assertEqual(vina.get_error(),
                             vina.get_dir_name() +
                             ' already exists and status is unknown')

            # vina already complete
            vina = AutoDockVinaTask(temp_dir, params)
            open(os.path.join(vina.get_dir(),
                              D3RTask.COMPLETE_FILE), 'a').close()
            self.assertEqual(vina.can_run(), False)
            self.assertEqual(vina.get_error(), None)

        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_can_run_is_false(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            # return immediately cause can_run is false
            vina = AutoDockVinaTask(temp_dir, params)
            vina.run()
            self.assertEqual(vina.get_error(),
                             'chimeraprep task has notfound status')
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_vina_not_set(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            proteinligprep = ChimeraProteinLigPrepTask(temp_dir, params)
            proteinligprep.create_dir()
            open(os.path.join(proteinligprep.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            vina = AutoDockVinaTask(temp_dir, params)
            vina.run()
            self.assertEqual(vina.get_error(),
                             'vina not set')
            # test files get created
            self.assertEqual(os.path.isdir(vina.get_dir()),
                             True)
            errfile = os.path.join(vina.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_vina_fails(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.vina = 'false'
            proteinligprep = ChimeraProteinLigPrepTask(temp_dir, params)
            proteinligprep.create_dir()
            open(os.path.join(proteinligprep.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            vina = AutoDockVinaTask(temp_dir, params)

            vina.run()
            self.assertEqual(vina.get_error(),
                             'Non zero exit code: 1 received. Standard out: ' +
                             ' Standard error: ')
            # test file gets created
            errfile = os.path.join(vina.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)

            stderr = os.path.join(vina.get_dir(),
                                  'false.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(vina.get_dir(),
                                  'false.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_fails_cause_vina_is_not_found(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.vina = '/bin/doesnotexist'
            proteinligprep = ChimeraProteinLigPrepTask(temp_dir, params)
            proteinligprep.create_dir()
            open(os.path.join(proteinligprep.get_dir(), D3RTask.COMPLETE_FILE),
                 'a').close()
            vina = AutoDockVinaTask(temp_dir, params)

            vina.run()
            self.assertEqual(vina.get_error(),
                             'Caught Exception trying to run ' +
                             '/bin/doesnotexist --structuredir ' +
                             proteinligprep.get_dir() + ' --outdir ' +
                             vina.get_dir() +
                             ' : [Errno 2] No such file or directory')

            # test files get created
            errfile = os.path.join(vina.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), True)
        finally:
            shutil.rmtree(temp_dir)

    def test_run_succeeds(self):
        temp_dir = tempfile.mkdtemp()
        try:
            params = D3RParameters()
            params.vina = 'true'
            proteinligprep = ChimeraProteinLigPrepTask(temp_dir, params)
            proteinligprep.create_dir()
            open(os.path.join(proteinligprep.get_dir(),
                              D3RTask.COMPLETE_FILE),
                 'a').close()
            vina = AutoDockVinaTask(temp_dir, params)

            vina.run()
            self.assertEqual(vina.get_error(), None)
            # test files get created
            errfile = os.path.join(vina.get_dir(),
                                   D3RTask.ERROR_FILE)
            self.assertEqual(os.path.isfile(errfile), False)

            compfile = os.path.join(vina.get_dir(),
                                    D3RTask.COMPLETE_FILE)
            self.assertEqual(os.path.isfile(compfile), True)
            stderr = os.path.join(vina.get_dir(),
                                  'true.stderr')
            self.assertEqual(os.path.isfile(stderr), True)
            stdout = os.path.join(vina.get_dir(),
                                  'true.stdout')
            self.assertEqual(os.path.isfile(stdout), True)
        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



__author__ = 'robswift'

import logging
from d3r.filter.filtering_sets import do_not_call
from d3r.blast.query import Query

logger = logging.getLogger(__name__)


def create_queries(polymer, non_polymer, ph):
    """
    Runs InPut.read_sequences and Input.read_ligands, which read in the sequence information contained in
    new_release_sequence.tsv and the ligand information contained in new_release_nonpolymer.tsv, respectively, and
    returns a list of d3r.blast.Target() objects.
    :param polymer: (string) the absolute path to new_release_sequence.tsv
    :param non_polymer: (string) the absolute path to new_release_nonpolymer.tsv
    :return: (d3r.blast.Target() object)
    """
    queries = read_sequences(polymer)
    if queries is not None:
        logger.debug('Found ' + str(len(queries)) + ' queries from ' +
                     polymer + ' file')

    read_ph(ph, queries)
    read_ligands(non_polymer, queries)
    return queries


def read_sequences(polymer):
    """
    Reads the information in a new_release_sequence.tsv file. A blast.Target() object is created for each unique wwPDB
    ID and appended to a list, which is returned.
    :param polymer: the absolute path to a new_release_sequence.tsv file
    :return: queries a list of query objects.
    """
    queries = []
    handle = open(polymer, 'r')
    for line in handle.readlines():
        if line.startswith('PDB_ID'):
            continue

        words = line.split()
        try:
            pdb_id = words[0].lower()
            chain_id = words[1]
            seq = words[2]
            queries = add_sequence(queries, seq, pdb_id, chain_id)
        except:
            logger.exception('Caught exception')
            continue
    handle.close()
    return queries


def add_sequence(queries, seq, pdb_id, chain_id):
    """
    Adds a FASTA sequence, with a specific chain and wwPDB ID, to the
    appropriate target object.
    :param queries: a list of target objects, which may be empty
    :param seq: a FASTA sequence. It will be converted to a Bio.SeqRecord
    :param pdb_id: The corresponding wwPDB id.
    :param chain_id: The corresponding chain id.
    """
    added = [query.pdb_id for query in queries if query]
    if pdb_id in added:
        queries[added.index(pdb_id)].set_sequence(chain_id, seq)
    else:
        query = Query()
        query.pdb_id = format(pdb_id.lower())
        query.set_sequence(chain_id, seq)
        queries.append(query)
    return queries


def read_ligands(non_polymer, queries):
    """
    Reads the information in a new_release_structure_nonpolymer.tsv file. The
    ligands are mapped to the appropriate
    target object by their wwPDB IDs. Morerover, each ligand is labeled as
    'do_not_call' or 'dock' depending on whether
    or not the resname of the ligand is found in the do_not_call set in the
    filter.filtering_sets module.
    :param non_polymer: absolute path to the pre-release non_polymer.tsv file
    :param queries, a list of target objects.
    """
    handle = open(non_polymer, 'r')
    for line in handle.readlines():
        if line.startswith('PDB_ID'):
            continue
        words = line.split()
        if words:
            try:
                pdb_id = words[0].lower()
                resname = words[1]
                inchi = words[2]
                ligand_label = label(resname)
                add_ligand(pdb_id, resname, inchi, ligand_label, queries)
            except:
                logger.exception('Caught exception')
                continue
    handle.close()


def add_ligand(pdb_id, resname, inchi, label, queries):
    """
    Adds the ligand, represented by its resname, inchi string, and label ( which can be 'dock' or 'do_not_call'), to
    the target with the corresponding wwPDB id.
    :param pdb_id: (string) wwPDBID
    :param resname: (string) the resname of the ligand.
    :param inchi: (string) the inchi string of the ligand
    :param queries: (list) a list of target objects
    :return:
    """
    added = [target.pdb_id for target in queries if target]
    queries[added.index(pdb_id)].set_ligand(resname, inchi, label)

def read_ph(ph, queries):
    """

    :param ph:
    :param queries:
    :return:
    """
    handle = open(ph, 'r')
    for line in handle.readlines():
        if line.startswith('PDB_ID'):
            continue
        words = line.split()
        if words:
            try:
                pdb_id = words[0].lower()
                exp_ph = words[1]
                add_ph(pdb_id, exp_ph, queries)
            except:
                logger.exception('Caught exception')
                continue
    handle.close()

def add_ph(pdb_id, exp_ph, queries):
    """

    :param pdb_id:
    :param exp_ph:
    :param queries:
    :return:
    """
    added = [query.pdb_id for query in queries if query]
    queries[added.index(pdb_id)].exp_ph = exp_ph

def label(resname):
    """
    Returns a label for the input resname. The label is either 'do_not_call' or 'dock', depending on whether the ligand
    is found in the do_not_call set in the filter.filtering_sets.
    :param resname:
    :return: ligand_label, a string
    """
    if resname and resname in do_not_call:
        ligand_label = 'do_not_call'
    else:
        ligand_label = 'dock'
    return ligand_label



__author__ = 'churas'

import os
import logging

from d3r.celpp.task import D3RTask
from d3r.celpp.proteinligprep import ProteinLigPrepTask

logger = logging.getLogger(__name__)


class GlideTask(D3RTask):
    """Performs glide docking

    """
    MAE_FILES = ['LMCSS' + os.sep + 'LMCSS_dock_pv.maegz',
                 'SMCSS' + os.sep + 'SMCSS_dock_pv.maegz',
                 'hiResApo' + os.sep + 'hiResApo_dock_pv.maegz',
                 'hiResHolo' + os.sep + 'hiResHolo_dock_pv.maegz']

    FINAL_LOG = 'final.log'
    PBDID_TXT_SUFFIX = '.txt'

    def __init__(self, path, args):
        super(GlideTask, self).__init__(path, args)
        self.set_name('glide')

        prep = ProteinLigPrepTask(path, args)
        self.set_stage(prep.get_stage() + 1)
        self.set_status(D3RTask.UNKNOWN_STATUS)

    def get_uploadable_files(self):
        """Returns list of files that can be uploaded to remote server

           List will contain these files if found on the file system
           plus stderr/stdout files

           final.log
           pbdid/LMCSS/LMCSS_dock_pv.maegz
           pbdid/SMCSS/SMCSS_dock_pv.maegz
           pbdid/hiResApo/hiResApo_dock_pv.maegz
           pbdid/hiResHolo/hiResHolo_dock_pv.maegz
           pbdid/pbdid.txt

           "returns: list of files that can be uploaded.
        """
        # get the stderr/stdout files
        file_list = super(GlideTask, self).get_uploadable_files()

        out_dir = self.get_dir()

        try:
            final_log = os.path.join(out_dir, GlideTask.FINAL_LOG)
            if os.path.isfile(final_log):
                file_list.append(final_log)

            for entry in os.listdir(out_dir):
                full_path = os.path.join(out_dir, entry)
                if not os.path.isdir(full_path):
                    continue

                pbdid = full_path + GlideTask.PBDID_TXT_SUFFIX
                logger.debug('Looking for file ' + pbdid)
                if os.path.isfile(pbdid):
                    file_list.append(pbdid)

                logger.debug('Looking for .maegz files in ' + full_path)
                for mae_name in GlideTask.MAE_FILES:
                    mae = os.path.join(full_path, mae_name)
                    if os.path.isfile(mae):
                        file_list.append(mae)
        except OSError:
            logger.exception('Caught exception looking for pbdid folders')
        return file_list

    def can_run(self):
        """Determines if task can actually run

           This method first verifies the `ProteinLigPrep` task
           has `D3RTask.COMPLETE_STATUS` for
           status.  The method then verifies a `GlideTask` does
           not already exist.  If above is not true then self.set_error()
           is set with information about the issue
           :return: True if can run otherwise False
        """
        self._can_run = False
        self._error = None
        # check blast
        blastnfilter = ProteinLigPrepTask(self._path, self._args)
        blastnfilter.update_status_from_filesystem()
        if blastnfilter.get_status() != D3RTask.COMPLETE_STATUS:
            logger.info('Cannot run ' + self.get_name() + 'task ' +
                        'because ' + blastnfilter.get_name() + 'task' +
                        'has a status of ' + blastnfilter.get_status())
            self.set_error(blastnfilter.get_name() + ' task has ' +
                           blastnfilter.get_status() + ' status')
            return False

        # check this task is not complete and does not exist

        self.update_status_from_filesystem()
        if self.get_status() == D3RTask.COMPLETE_STATUS:
            logger.debug("No work needed for " + self.get_name() +
                         " task")
            return False

        if self.get_status() != D3RTask.NOTFOUND_STATUS:
            logger.warning(self.get_name() + " task was already " +
                           "attempted, but there was a problem")
            self.set_error(self.get_dir_name() + ' already exists and ' +
                           'status is ' + self.get_status())
            return False
        self._can_run = True
        return True

    def run(self):
        """Runs ProteinLigPrepTask after verifying proteinligprep was good

           Method requires can_run() to be called before hand with
           successful outcome
           Otherwise method invokes D3RTask.start then this method
           creates a directory and invokes blastnfilter script.  Upon
           completion results are analyzed and success or error status
           is set appropriately and D3RTask.end is invoked
           """
        super(GlideTask, self).run()

        if self._can_run is False:
            logger.debug(
                self.get_dir_name() + ' cannot run cause _can_run flag '
                                      'is False')
            return

        try:
            logger.debug('glide set to ' +
                         self.get_args().glide)
        except AttributeError:
            self.set_error('glide not set')
            self.end()
            return

        proteinligprep = ProteinLigPrepTask(self._path, self._args)

        #
        # glide.py --structuredir <path to stage.3.proteinligprep> \
        # --outdir <path to stage.4.glide>
        #
        cmd_to_run = (self.get_args().glide + ' --structuredir ' +
                      proteinligprep.get_dir() +
                      ' --outdir ' + self.get_dir())

        glide_name = os.path.basename(self.get_args().glide)

        self.run_external_command(glide_name, cmd_to_run,
                                  True)
        # assess the result
        self.end()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'churas'


import unittest
import os
import tempfile
import shutil


"""
test_participant
----------------------------------

Tests for `participant` module.
"""

from d3r.celpp.participant import Participant
from d3r.celpp.participant import ParticipantDatabase
from d3r.celpp.participant import ParticipantDatabaseFromCSVFactory


class TestParticipant(unittest.TestCase):
    def setUp(self):
        pass

    def test_participant(self):
        p = Participant(None, None, None, None)
        self.assertEqual(p.get_d3rusername(), None)
        self.assertEqual(p.get_email(), None)
        self.assertEqual(p.get_guid(), None)
        self.assertEqual(p.get_name(), None)

        p = Participant('name', 'd3rusername', 'guid', 'email@email.com')
        self.assertEqual(p.get_d3rusername(), 'd3rusername')
        self.assertEqual(p.get_email(), 'email@email.com')
        self.assertEqual(p.get_guid(), 'guid')
        self.assertEqual(p.get_name(), 'name')
        self.assertEqual(p.get_priority(), 0)

        p = Participant('2name', '2d3rusername', '2guid', '2email@email.com',
                        priority=234)
        self.assertEqual(p.get_d3rusername(), '2d3rusername')
        self.assertEqual(p.get_email(), '2email@email.com')
        self.assertEqual(p.get_guid(), '2guid')
        self.assertEqual(p.get_name(), '2name')
        self.assertEqual(p.get_priority(), 234)

    def test_participant_database(self):

        pdb = ParticipantDatabase(None)
        self.assertEqual(pdb.get_participants(), None)
        self.assertEqual(pdb.get_participant_by_guid('bob'), None)

        pdb = ParticipantDatabase([])
        self.assertEqual(len(pdb.get_participants()), 0)
        self.assertEqual(pdb.get_participant_by_guid('bob'), None)
        self.assertEqual(pdb.get_participant_by_guid('bob',
                                                     exact_match=True), None)
        self.assertEqual(pdb.get_participant_by_guid('guidx'), None)
        self.assertEqual(pdb.get_participant_by_guid('xguid'), None)
        self.assertEqual(pdb.get_participant_by_guid('GUID'), None)

        plist = [Participant('1name', '1d3rusername', 'guid',
                             '1email@email.com'),
                 Participant('2name', '2d3rusername', 'guidy',
                             '2email@email.com'),
                 Participant('3name', '3d3rusername', '12345',
                             '3email@email.com'),
                 Participant('4name', '4d3rusername', '567890_foo',
                             '4email@email.com')]

        pdb = ParticipantDatabase(plist)
        self.assertEqual(pdb.get_participant_by_guid(None), None)
        self.assertEqual(pdb.get_participant_by_guid('bob'), None)
        self.assertEqual(pdb.get_participant_by_guid('guidx'), None)
        self.assertEqual(pdb.get_participant_by_guid('xguid'), None)
        self.assertEqual(pdb.get_participant_by_guid('GUID'), None)

        p = pdb.get_participant_by_guid('guid')
        self.assertEqual(p.get_name(), '1name')

        p = pdb.get_participant_by_guid('guidy')
        self.assertEqual(p.get_name(), '2name')

        p = pdb.get_participant_by_guid('12345')
        self.assertEqual(p.get_name(), '3name')

        p = pdb.get_participant_by_guid('12345', exact_match=True)
        self.assertEqual(p.get_name(), '3name')

        p = pdb.get_participant_by_guid('12345_2')
        self.assertEqual(p.get_name(), '3name')

        p = pdb.get_participant_by_guid('12345_2', exact_match=False)
        self.assertEqual(p.get_name(), '3name')

        p = pdb.get_participant_by_guid('12345_mydock', exact_match=False)
        self.assertEqual(p.get_name(), '3name')

        p = pdb.get_participant_by_guid('12345_my-dock', exact_match=False)
        self.assertEqual(p.get_name(), '3name')

        # Suffixes can't contain underscores
        p = pdb.get_participant_by_guid('12345_my_dock', exact_match=False)
        self.assertEqual(p, None)

        p = pdb.get_participant_by_guid('12345_2', exact_match=True)
        self.assertEqual(p, None)

        p = pdb.get_participant_by_guid('12345_2_b')
        self.assertEqual(p, None)

        # test a longer guid works
        p = pdb.get_participant_by_guid('567890_foo')
        self.assertEqual(p.get_name(), '4name')

        p = pdb.get_participant_by_guid('4567890_foo')
        self.assertEqual(p, None)

    def test_participant_database_from_csv_factory(self):
        temp_dir = tempfile.mkdtemp()
        try:

            csvfile = os.path.join(temp_dir, 'participant_list.csv')

            # parse None as file
            pfac = ParticipantDatabaseFromCSVFactory(None)
            self.assertEqual(pfac.get_participant_database(), None)

            # parse non existant file
            pfac = ParticipantDatabaseFromCSVFactory(csvfile)
            self.assertEqual(pfac.get_participant_database(), None)

            # parse empty file
            open(csvfile, 'a').close()
            pfac = ParticipantDatabaseFromCSVFactory(csvfile)
            pdb = pfac.get_participant_database()
            self.assertEqual(len(pdb.get_participants()), 0)

            # parse file with only header
            f = open(csvfile, 'w')
            f.write('name,d3rusername,guid,email\n')
            f.flush()
            f.close()
            pfac = ParticipantDatabaseFromCSVFactory(csvfile)
            pdb = pfac.get_participant_database()
            self.assertEqual(len(pdb.get_participants()), 0)

            # parse file with header and 1 valid entry
            f = open(csvfile, 'w')
            f.write('name,d3rusername,guid,email\n')
            f.write('joe,jj,123,j@j.com\n')
            f.flush()
            f.close()
            pfac = ParticipantDatabaseFromCSVFactory(csvfile)
            pdb = pfac.get_participant_database()
            self.assertEqual(len(pdb.get_participants()), 1)
            p = pdb.get_participant_by_guid('123')
            self.assertEqual(p.get_email(), 'j@j.com')

            # parse file with no header and 1 valid entry
            f = open(csvfile, 'w')
            f.write('joe,jj,123,j@j.com\n')
            f.flush()
            f.close()
            pfac = ParticipantDatabaseFromCSVFactory(csvfile)
            pdb = pfac.get_participant_database()
            self.assertEqual(len(pdb.get_participants()), 1)
            p = pdb.get_participant_by_guid('123')
            self.assertEqual(p.get_email(), 'j@j.com')

            # parse file with header and 2 valid and 1 invalid entries

            f = open(csvfile, 'w')
            f.write('name,d3rusername,guid,email\n')
            f.write('joe,jj,123,j@j.com\n')
            f.write('uhoh,2,many,commas,uhoh@email.com,foo\n')
            f.write('phil,pp,456,p@p.com\n')

            f.flush()
            f.close()
            pfac = ParticipantDatabaseFromCSVFactory(csvfile)
            pdb = pfac.get_participant_database()
            self.assertEqual(len(pdb.get_participants()), 2)
            p = pdb.get_participant_by_guid('123')
            self.assertEqual(p.get_email(), 'j@j.com')
            p = pdb.get_participant_by_guid('456')
            self.assertEqual(p.get_email(), 'p@p.com')

            # parse file with header and 2 valid entries both with
            # priorities and 1 valid entry except invalid priority
            f = open(csvfile, 'w')
            f.write('name,d3rusername,guid,email,priority\n')
            f.write('joe,jj,123,j@j.com,4\n')
            f.write('xx,xxx,1234,j@j.com,foo\n')
            f.write('phil,pp,456,p@p.com,6\n')

            f.flush()
            f.close()
            pfac = ParticipantDatabaseFromCSVFactory(csvfile)
            pdb = pfac.get_participant_database()
            self.assertEqual(len(pdb.get_participants()), 3)
            p = pdb.get_participant_by_guid('123')
            self.assertEqual(p.get_email(), 'j@j.com')
            self.assertEqual(p.get_priority(), 4)
            p = pdb.get_participant_by_guid('456')
            self.assertEqual(p.get_email(), 'p@p.com')
            self.assertEqual(p.get_priority(), 6)
            p = pdb.get_participant_by_guid('1234')
            self.assertEqual(p.get_email(), 'j@j.com')
            self.assertEqual(p.get_priority(), 0)

            # parse file with carriage returns
            f = open(csvfile, 'w')
            f.write('name,d3rusername,guid,email\r')
            f.write('joe bob , jb,12345,j@j.com\r')
            f.write('J W,ha@ha.com,33567 , ha@ha.com')
            f.flush()
            f.close()
            pdb = pfac.get_participant_database()
            self.assertEqual(len(pdb.get_participants()), 2)
            p = pdb.get_participant_by_guid('12345')
            self.assertEqual(p.get_email(), 'j@j.com')
            self.assertEqual(p.get_name(), 'joe bob')
            self.assertEqual(p.get_d3rusername(), 'jb')
            p = pdb.get_participant_by_guid('33567')
            self.assertEqual(p.get_email(), 'ha@ha.com')
            self.assertEqual(p.get_name(), 'J W')
            self.assertEqual(p.get_d3rusername(), 'ha@ha.com')

        finally:
            shutil.rmtree(temp_dir)

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import os
import logging
import re
import urllib
import gzip
import subprocess
import shlex
import time
import urllib2
from dateutil.parser import parse
from datetime import date
from datetime import datetime
from datetime import timedelta
from dateutil.tz import tzlocal


logger = logging.getLogger(__name__)

DAYS_IN_WEEK = 7

FRIDAY_WEEKDAY = 4

DATA_SET_WEEK_PREFIX = 'dataset.week.'

LOG_FORMAT = "%(asctime)-15s %(levelname)s %(name)s %(message)s"


class DownloadError(Exception):
    """Exception to denote error downloading data
    """
    pass


def get_all_celpp_years(celppdir):
    """Given a directory find all year directories within

       This method looks for all directories within `celppdir`
       that fit the structure of a year directory ####
       or the regex ^\d\d\d\d$.
       Full paths to those directories are returned
       :return: List year directories, note path is NOT prefixed
       :raises: Exception if celppdir is not a directory
    """
    if not os.path.isdir(celppdir):
        raise Exception(celppdir + " is not a directory")

    year_list = []
    dir_pattern = re.compile("^\d\d\d\d$")
    for entry in os.listdir(celppdir):
        if re.match(dir_pattern, entry):
            full_path = os.path.join(celppdir, entry)
            if os.path.isdir(full_path):
                year_list.append(entry)

    return year_list


def find_latest_year(celppdir):
    """Given a directory find the latest year

       The latest year will be a folder of 4 digits
       and have the highest value ie 2015
       :return: Directory or None if none is found
       :raises: Exception: If celppdir is not a directory
       """

    if not os.path.isdir(celppdir):
        raise Exception(celppdir + " is not a directory")

    latest_year = -1
    latest_entry = None
    full_path = None
    for entry in get_all_celpp_years(celppdir):
            entry_year = int(entry)
            if entry_year > latest_year:
                full_path = os.path.join(celppdir, entry)
                if os.path.isdir(full_path):
                    latest_year = entry_year
                    latest_entry = full_path

    if latest_year == -1:
        return

    return latest_entry


def get_celpp_year_from_path(dir):
    """Given path extract the year number from it

       Expects path of
       /..../year/dataset.week.#
       or year/dataset.week.#

    :param dir: full or partial path to week
    :return year #### from path or '0' if year extracted is not a 4
    digit number
    """
    if dir is None:
        raise Exception('Path is none')

    remove_week = re.sub("\/" + DATA_SET_WEEK_PREFIX + ".*$", "", dir)
    year = re.sub("^.*\/", "", remove_week)

    dir_pattern = re.compile("^\d\d\d\d$")

    if not re.match(dir_pattern, year):
        logger.warning('Year ( ' + year + ' ) extracted from ' + dir +
                       ' is not a 4 digit number')
        return '0'
    return year


def get_all_celpp_weeks(celpp_year_dir):
    """Given a celpp year directory get all week directories

       Expects celpp_year_dir to be full path to year dir ie ####/
       and finds all directories of format dataset.week.#
       returning a list of the week numbers ie #
       :returns: List of week numbers
       :raises Exception: If celpp_year_dir is not a directory
    """
    if not os.path.isdir(celpp_year_dir):
        raise Exception(celpp_year_dir + " is not a directory")

    week_list = []

    dir_pattern = re.compile("^" + DATA_SET_WEEK_PREFIX + "\d+$")

    for entry in os.listdir(celpp_year_dir):
        if re.match(dir_pattern, entry):
            weeknum = re.sub(DATA_SET_WEEK_PREFIX, "", entry)
            full_path = os.path.join(celpp_year_dir, entry)
            if os.path.isdir(full_path):
                week_list.append(weeknum)

    return week_list


def find_latest_weekly_dataset(celppdir):
    """Given a directory find the latest dataset

       This method looks in directory passed in for
       paths with dataset.week.# format and returns
       the path with highest # on it.  If none are
       found then None is returned.
       :return: Directory upon success or None if no directory found
       :raises: Exception: If celppdir is not a directory
       """
    latest_year = find_latest_year(celppdir)

    if latest_year is None:
        return

    latest_entry = None
    latest_weeknum = -1
    full_path = None
    for entry in get_all_celpp_weeks(latest_year):
            weeknum = int(entry)
            if weeknum > latest_weeknum:
                full_path = os.path.join(latest_year,
                                         DATA_SET_WEEK_PREFIX + entry)
                if os.path.isdir(full_path):
                    latest_weeknum = weeknum
                    latest_entry = full_path

    return latest_entry


def get_celpp_week_number_from_path(dir):
    """Given path extract the week number from it

       Expects either path of
       /..../year/dataset.week.#
       or dataset.week.#

    :param dir: full or partial path to week
    :return the # from dataset.week.# in `dir` path or '0' if value is
    not a 1-2 digit number
    """
    if dir is None:
        raise Exception('Path is none')

    weeknum = re.sub("^.*" + DATA_SET_WEEK_PREFIX, "", dir)

    dir_pattern = re.compile("^\d\d?$")
    if not re.match(dir_pattern, weeknum):
        logger.warning('Week number ( ' + weeknum + ' ) extracted from path '
                       + dir + ' is not a 2 digit number')
        return '0'
    return weeknum


def get_previous_friday_from_date(the_datetime):
    """Given a date, function finds date of previous Friday
       Given `the_date` code finds the date of the previous
       Friday with same hour, minute, and second.
       :return: datetime object containing date of previous Friday
       :raises: Exception if `the_date` is None
    """
    if the_datetime is None:
        raise Exception('Must pass a valid datetime')
    assert isinstance(the_datetime, datetime)

    weekday = the_datetime.weekday()
    if weekday == FRIDAY_WEEKDAY:
        return the_datetime

    if weekday > FRIDAY_WEEKDAY:
        tdelta = timedelta(days=(weekday - FRIDAY_WEEKDAY))
    else:
        if weekday < FRIDAY_WEEKDAY:
            tdelta = timedelta(days=(3 + weekday))

    return the_datetime - tdelta


def is_datetime_after_celpp_week_start(the_datetime):
    """Determines if `the_datetime` is after celpp week start
    :returns: True if yes otherwise False
    :raises Exception: If `the_datetime` is not a valid datetime object
    """
    if the_datetime is None:
        raise Exception('Must pass a valid datetime')
    assert isinstance(the_datetime, datetime)

    prev_friday = get_previous_friday_from_date(datetime.now(tzlocal()))

    if the_datetime >= prev_friday:
        return True

    return False


def has_url_been_updated_since_start_of_celpp_week(url):
    """Checks if url has been updated since start of celpp week
    """
    if url is None:
        raise Exception('url cannot be None')

    req = urllib2.Request(url)
    resp = urllib2.urlopen(req)

    last_modified = resp.info()['Last-Modified']
    logger.debug(url + ' last modified: ' + last_modified)
    lmdate = parse(last_modified)
    return is_datetime_after_celpp_week_start(lmdate)


def get_celpp_week_of_year_from_date(the_date):
    """ Returns CELPP week of year
        The CELPP week of year matches ISO week of year, but it
        starts on Friday instead of Monday.  The way it works is
        the code examines `the_date` and if its 12:00am Monday -
        11:59pm Thursday the current week according to date.isocalendar()
        is used.  If date falls between Friday 12:00am to Sunday 11:59pm
        this function advances the date to the following Monday and extracts
        week of year.
        :return: tuple with first value the week and second value the year
        :raises: Exception if `the_date` is None
    """
    if the_date is None:
        raise Exception('Must pass a valid date')
    assert isinstance(the_date, date)
    updated_date = the_date

    day_of_week = the_date.weekday()
    if day_of_week >= 4:
        updated_date = the_date + timedelta(days=(DAYS_IN_WEEK - day_of_week))
        logger.debug('Date ' + str(the_date) + ' is between Friday and ' +
                     'Sunday shifting date to ' + str(updated_date))

    return (updated_date.isocalendar()[1],
            updated_date.isocalendar()[0])


def create_celpp_week_dir(celpp_week_tuple, celppdir):
    """Creates celpp week dir in `celppdir`
    Creates celpp week directory under `celppdir`
    using `celpp_week_tuple` to get year and week of year
    diretories created:
    celppdir/<celpp_week_tuple[1]>/dataset.week.<celpp_week_tuple[0]>
    If directory already exists no operation is performed.
    :raises: OSError if unable to create directory
    """
    if celpp_week_tuple is None:
        raise Exception('celpp week tuple must be set')

    if celppdir is None:
        raise Exception('celppdir must be set')

    dir_to_create = os.path.join(celppdir, str(celpp_week_tuple[1]),
                                 'dataset.week.'+str(celpp_week_tuple[0]))
    if os.path.isdir(dir_to_create):
        logger.debug(dir_to_create +
                     ' directory already exists skipping creation')
        return

    logger.debug('Creating ' + dir_to_create)
    os.makedirs(dir_to_create, mode=0o775)


def download_url_to_file(url, download_path, num_retries,
                         retry_sleep_time_secs):
    """Downloads `url` to path specified by `download_path`

    Downloads `url` with built in retry `num_retries` and sleep
    `retry_sleep_time_secs` between retries
    :param url: url to download file from
    :param download_path: full path to file to write output to
    :param num_retries: number of retries, 0 means none, 1 means one retry
    :param retry_sleep_time_secs: # seconds delay between retry, cannot
                                  be negative
    :raises: DownloadError if there was an error downloading the file or
             one of the parameters has invalid value
    """

    if url is None:
        raise DownloadError('url is not set')

    if download_path is None:
        raise DownloadError('download_path is not set')

    if num_retries is None:
        num_retries = 0

    if retry_sleep_time_secs is None:
        retry_sleep_time_secs = 0

    if retry_sleep_time_secs < 0:
        raise DownloadError('retry sleep time cannot be negative')

    count = 0
    while count <= num_retries:
        logger.debug('Try # ' + str(count) + ' of ' +
                     str(num_retries) + ' to download ' +
                     download_path + ' from ' + url)
        try:
            (f, info) = urllib.urlretrieve(url, download_path)
            return
        except Exception:
            logger.exception('Caught exception trying to download file')
        count += 1
        logger.debug('Download failed, sleeping ' +
                     str(retry_sleep_time_secs) +
                     'seconds')
        time.sleep(retry_sleep_time_secs)

    raise DownloadError('Unable to download file from ' +
                        url + ' to ' + download_path)


def gunzip_file(gzip_file, dest_file):
    """Uses Python gzip library to uncompress gzip file

       :param gzip_file: path to file to gunzip
       :param dest_file: gunzip destination file
       :raises IOError: If there was a problem with operation
    """
    f_in = gzip.open(gzip_file, 'rb')
    f_out = open(dest_file, 'wb')
    for f in f_in:
        f_out.write(f)
    f_out.flush()
    f_out.close()
    f_in.close()


def append_string_to_file(dest_file, the_string):
    """Appends text in `the_string` to file set by `dest_file`

       Opens file specified by `dest_file` in append mode and
       writes `the_string` contents to the file.  Function then
       flushes and closes file.

       :param dest_file: File to append data to
       :param the_string: Data to append to file
       :raises IOError: If there is an issue writing data to the file
       :raises TypeError: If `dest_file` is None
    """
    f = open(dest_file, 'a')
    f.write(the_string)
    f.flush()
    f.close()


def get_file_line_count(the_file):
    """Counts number of lines in file

       Opens file and counts number of lines in
       file
       :param the_file: file to examine
       :raises TypeError: if `the_file` is None
       :raises IOError: if there is an issue reading the file
    """
    f = open(the_file, 'r')
    counter = 0
    for line in f:
        counter += 1
    f.close()
    return counter


def run_external_command(cmd_to_run):
    """Runs command via external process
       Executes command in `cmd_to_run` which should
       be a single string command ie: ls -la or /bin/true
       :param cmd_to_run: The command with arguments to run
       :raises: All exceptions from subprocess.Popen()
       :returns: tuple (exitcode, stdout, stderr)
    """

    if cmd_to_run is None:
        return 256, '', 'Command must be set'

    logger.info("Running command " + cmd_to_run)

    p = subprocess.Popen(shlex.split(cmd_to_run),
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)

    out, err = p.communicate()
    return p.returncode, out, err


def run_external_command_with_timeout(cmd_to_run, tmp_dir,
                                      timeout=None,
                                      kill_delay=10,
                                      polling_sleep_time=1):
    """Runs command passed in
    :param cmd_to_run: command with arguments to run set as a string
    :param tmp_dir: temporary directory where stdout.txt and stderr.txt
                    files can be temporarily written
    :param timeout: time in seconds to allow process to run before killing
    :param polling_sleep_time: time in seconds to sleep between checks for
                               command completion.
    :returns: tuple containing (exit code, stdout, stderr)
    """
    if cmd_to_run is None:
        return 256, '', 'Command must be set'

    if tmp_dir is None:
        return 255, '', 'Tmpdir must be set'

    if not os.path.isdir(tmp_dir):
        return 254, '', 'Tmpdir must be a directory'

    logger.info("Running command " + cmd_to_run)
    tmp_stdout = os.path.join(tmp_dir, 'stdout.txt')
    logger.debug('stdout file: ' + tmp_stdout)

    stdout_f = open(tmp_stdout, 'w')
    tmp_stderr = os.path.join(tmp_dir, 'stderr.txt')
    logger.debug('stderr file: ' + tmp_stderr)

    stderr_f = open(tmp_stderr, 'w')
    p = subprocess.Popen(shlex.split(cmd_to_run),
                         stdout=stdout_f,
                         stderr=stderr_f)

    logger.debug('Waiting for process to complete')
    p.poll()
    logger.debug('Polling returned None, looping until'
                 ' p.returncode is not None')

    if timeout is not None:
        start_time = time.time()
    else:
        start_time = 0

    terminate_invoked = False

    while p.returncode is None:
        if timeout is not None and int(time.time() - start_time) > timeout:
            if terminate_invoked is False:
                logger.info('Timeout of ' + str(timeout) +
                            'exceeded invoking terminate')
                p.terminate()

                # pushing start time into future
                # to give child process a chance to
                # exit when SIGTERM is called
                start_time += kill_delay

                terminate_invoked = True
            else:
                logger.info('Timeout of ' + str(timeout + kill_delay) +
                            'exceeded invoking kill')
                p.kill()
        time.sleep(polling_sleep_time)
        p.poll()

    logger.debug('Got a return code that is not None: ' +
                 str(p.returncode))
    stdout_f.flush()
    stdout_f.close()
    stderr_f.flush()
    stderr_f.close()
    logger.debug('Writing stderr and stdout to ' + tmp_dir)
    fo = open(tmp_stdout, 'r')
    out = fo.read()
    fo.close()
    fe = open(tmp_stderr, 'r')
    err = fe.read()
    fe.close()

    return p.returncode, out, err


def setup_logging(theargs):
    """Sets up the logging for application

    Loggers are setup for:
    d3r.blastnfilter
    d3r.celpprunner
    d3r.celpp.blastnfilter
    d3r.celpp.dataimport
    d3r.celpp.glide
    d3r.celpp.makeblastdb
    d3r.celpp.proteinligprep
    d3r.celpp.evaluation
    d3r.celpp.task
    d3r.celpp.util
    d3r.celpp.uploader
    d3r.celpp.chimeraprep
    d3r.celpp.participant
    d3r.blast.ligand
    d3r.blast.hit
    d3r.blast.hit_sequence
    d3r.blast.query
    d3r.filter.filter
    d3r.utilities.analysis
    d3r.utilities.in_put
    d3r.utilities.run

    NOTE:  If new modules are added please add their loggers to this
    function

    The loglevel is set by theargs.loglevel and the format is set
    by LOG_FORMAT set at the top of this module.
    :param: theargs should have .loglevel set to one of the
    following strings: DEBUG, INFO, WARNING, ERROR, CRITICAL
    """
    theargs.logformat = LOG_FORMAT
    theargs.numericloglevel = logging.NOTSET
    if theargs.loglevel == 'DEBUG':
        theargs.numericloglevel = logging.DEBUG
    if theargs.loglevel == 'INFO':
        theargs.numericloglevel = logging.INFO
    if theargs.loglevel == 'WARNING':
        theargs.numericloglevel = logging.WARNING
    if theargs.loglevel == 'ERROR':
        theargs.numericloglevel = logging.ERROR
    if theargs.loglevel == 'CRITICAL':
        theargs.numericloglevel = logging.CRITICAL

    logger.setLevel(theargs.numericloglevel)
    logging.basicConfig(format=theargs.logformat)

    # There should be a line below for every package aka .py file
    # under celpp module
    logging.getLogger('d3r.blastnfilter')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpprunner')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celppreports')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.blastnfilter')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.challengedata')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.dataimport').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.glide').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.vina').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.makeblastdb')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.proteinligprep')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.evaluation').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.task').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.util').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.filetransfer')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.chimeraprep')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.participant')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.celpp.postevaluation')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.blast.ligand').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.blast.hit').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.blast.hit_sequence')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.blast.query').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.filter.filter').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.utilities.analysis')\
        .setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.utilities.in_put').setLevel(theargs.numericloglevel)
    logging.getLogger('d3r.utilities.run').setLevel(theargs.numericloglevel)
